@@ -1,14 +1,15 @@
 # This file is part of Xpra.
-# Copyright (C) 2010-2017 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2010-2018 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 from xpra.client.client_widget_base import ClientWidgetBase
+from xpra.client.window_backing_base import WindowBackingBase
+from xpra.gtk_common.gobject_compat import import_glib
+from xpra.os_util import memoryview_to_bytes, _buffer
 from xpra.log import Logger
 log = Logger("tray")
 
-from xpra.client.gtk_base.gtk_window_backing_base import GTKWindowBacking
-
 
 class ClientTray(ClientWidgetBase):
     """
@@ -20,9 +21,11 @@ class ClientTray(ClientWidgetBase):
     DEFAULT_SIZE = [64, 64]
     DEFAULT_GEOMETRY = DEFAULT_LOCATION + DEFAULT_SIZE
 
-    def __init__(self, client, wid, w, h, tray_widget, mmap_enabled, mmap_area):
+    def __init__(self, client, wid, w, h, metadata, tray_widget, mmap_enabled, mmap_area):
         log("ClientTray%s", (client, wid, w, h, tray_widget, mmap_enabled, mmap_area))
-        ClientWidgetBase.__init__(self, client, wid, True)
+        ClientWidgetBase.__init__(self, client, 0, wid, True)
+        self._metadata = metadata
+        self.title = metadata.strget("title", "")
         self.tray_widget = tray_widget
         self._geometry = None
         self._window_alpha = True
@@ -34,6 +37,11 @@ class ClientTray(ClientWidgetBase):
         self.new_backing(w, h)
         self.idle_add(self.reconfigure)
 
+    def set_alpha(self):
+        #nothing to do,
+        #trays aren't really windows and transparency is always supported
+        pass
+
     def get_backing_class(self):
         return TrayBacking
 
@@ -126,7 +134,7 @@ class ClientTray(ClientWidgetBase):
     def update_metadata(self, metadata):
         log("%s.update_metadata(%s)", self, metadata)
 
-    def update_icon(self, width, height, coding, data):
+    def update_icon(self, img):
         #this is the window icon... not the tray icon!
         pass
 
@@ -158,6 +166,9 @@ class ClientTray(ClientWidgetBase):
         has_alpha = enc=="rgb32"
         tw = self.tray_widget
         if tw:
+            #some tray implementations can't deal with memoryviews..
+            if isinstance(pixels, (memoryview, _buffer or bytearray, bytearray)):
+                pixels = memoryview_to_bytes(pixels)
             tw.set_icon_from_data(pixels, has_alpha, w, h, rowstride, options)
 
 
@@ -168,10 +179,10 @@ class ClientTray(ClientWidgetBase):
             tw.cleanup()
 
     def __repr__(self):
-        return "ClientTray(%s)" % self._id
+        return "ClientTray(%i:%s)" % (self._id, self.title)
 
 
-class TrayBacking(GTKWindowBacking):
+class TrayBacking(WindowBackingBase):
     """
         This backing only stores the rgb pixels so
         we can use them with the real widget.
@@ -182,9 +193,10 @@ class TrayBacking(GTKWindowBacking):
     RGB_MODES = ["RGBA", "RGBX"]
     HAS_ALPHA = True
 
-    def __init__(self, wid, w, h, has_alpha, data=None):
+    def __init__(self, wid, _w, _h, _has_alpha, data=None):
         self.data = data
-        GTKWindowBacking.__init__(self, wid, True)
+        WindowBackingBase.__init__(self, wid, True)
+        self.idle_add = import_glib().idle_add
         self._backing = object()    #pretend we have a backing structure
 
     def get_encoding_properties(self):
