@@ -1,6 +1,6 @@
 # This file is part of Xpra.
 # Copyright (C) 2008 Nathaniel Smith <njs@pobox.com>
-# Copyright (C) 2012-2014 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2012-2018 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -20,6 +20,7 @@ except ImportError as e:
     log.warn("Warning: failed to load the gtk3 cairo workaround:")
     log.warn(" %s", e)
     log.warn(" rendering will be slow!")
+    del e
     set_image_surface_data = None
 
 
@@ -37,25 +38,26 @@ Instead we have to use PIL to convert via a PNG or Pixbuf!
 """
 class CairoBacking(CairoBackingBase):
 
-    RGB_MODES = ["RGBA", "RGBX", "RGB"]
+    RGB_MODES = ["BGRA", "BGRX", "RGBA", "RGBX", "BGR", "RGB"]
 
     def __repr__(self):
         return "gtk3.CairoBacking(%s)" % self._backing
 
-    def paint_jpeg(self, img_data, x, y, width, height, options, callbacks):
-        #don't use the native decoder... which would give us RGB data
-        return self.paint_image("jpeg", img_data, x, y, width, height, options, callbacks)
-
     def _do_paint_rgb(self, cairo_format, has_alpha, img_data, x, y, width, height, rowstride, options):
         """ must be called from UI thread """
         log("cairo._do_paint_rgb(%s, %s, %s %s,%s,%s,%s,%s,%s,%s) set_image_surface_data=%s, use pixbuf=%s", FORMATS.get(cairo_format, cairo_format), has_alpha, len(img_data), type(img_data), x, y, width, height, rowstride, options, set_image_surface_data, CAIRO_USE_PIXBUF)
         rgb_format = options.strget(b"rgb_format", "RGB")
-        #this format we can handle with the workaround:
-        if cairo_format==cairo.FORMAT_RGB24 and rgb_format=="RGB" and set_image_surface_data and not CAIRO_USE_PIXBUF:
-            img_surface = cairo.ImageSurface(cairo_format, width, height)
-            set_image_surface_data(img_surface, rgb_format, img_data, width, height, rowstride)
-            self.cairo_paint_surface(img_surface, x, y, options)
-            return True
+        if set_image_surface_data and not CAIRO_USE_PIXBUF:
+            if cairo_format==cairo.FORMAT_RGB24 and rgb_format in ("RGB", "BGR"):
+                img_surface = cairo.ImageSurface(cairo_format, width, height)
+                set_image_surface_data(img_surface, rgb_format, img_data, width, height, rowstride)
+                self.cairo_paint_surface(img_surface, x, y, options)
+                return True
+            if cairo_format==cairo.FORMAT_ARGB32 and rgb_format in ("RGBX", "BGRX"):
+                img_surface = cairo.ImageSurface(cairo_format, width, height)
+                set_image_surface_data(img_surface, rgb_format, img_data, width, height, rowstride)
+                self.cairo_paint_surface(img_surface, x, y, options)
+                return True
 
         if rgb_format in ("RGB", "RGBA", "RGBX"):
             data = GLib.Bytes(img_data)
