@@ -14,7 +14,7 @@ from gi.repository import GdkPixbuf             #@UnresolvedImport @UnusedImport
 from xpra.client.gtk3.cairo_backing import CairoBacking
 from xpra.client.gtk_base.gtk_client_window_base import GTKClientWindowBase, HAS_X11_BINDINGS
 from xpra.gtk_common.gtk_util import WINDOW_NAME_TO_HINT, WINDOW_EVENT_MASK, BUTTON_MASK
-from xpra.os_util import bytestostr
+from xpra.os_util import bytestostr, WIN32
 from xpra.log import Logger
 log = Logger("gtk", "window")
 paintlog = Logger("paint")
@@ -39,7 +39,7 @@ GTK3_OR_TYPE_HINTS = (Gdk.WindowTypeHint.DIALOG,
 """
 GTK3 version of the ClientWindow class
 """
-class ClientWindow(GTKClientWindowBase, Gtk.Window):
+class ClientWindow(GTKClientWindowBase):
 
     __gsignals__ = GTKClientWindowBase.__common_gsignals__
 
@@ -65,19 +65,19 @@ class ClientWindow(GTKClientWindowBase, Gtk.Window):
         # they moved it gobject, then removed it, unbelievable:
         # https://bugzilla.gnome.org/show_bug.cgi?id=641944
         #self.set_data("_kde_no_window_grab", 1)
-        def motion(w, event):
+        def motion(_w, event):
             self.do_motion_notify_event(event)
             return True
         self.connect("motion-notify-event", motion)
-        def press(w, event):
+        def press(_w, event):
             self.do_button_press_event(event)
             return True
         self.connect("button-press-event", press)
-        def release(w, event):
+        def release(_w, event):
             self.do_button_release_event(event)
             return True
         self.connect("button-release-event", release)
-        def scroll(w, event):
+        def scroll(_w, event):
             self.do_scroll_event(event)
             return True
         self.connect("scroll-event", scroll)
@@ -88,8 +88,19 @@ class ClientWindow(GTKClientWindowBase, Gtk.Window):
     def enable_alpha(self):
         screen = self.get_screen()
         visual = screen.get_rgba_visual()
+        #we can't do alpha on win32 with plain GTK,
+        #(though we handle it in the opengl backend)
+        if WIN32:
+            l = log
+        else:
+            l = log.error
         if visual is None or not screen.is_composited():
-            log.error("enable_alpha() cannot handle window transparency on screen %s", screen)
+            l("Error: cannot handle window transparency")
+            if visual is None:
+                l(" no RGBA visual")
+            else:
+                assert not screen.is_composited()
+                l(" screen is not composited")
             return False
         log("enable_alpha() using rgba visual %s for wid %s", visual, self._id)
         self.set_visual(visual)
@@ -152,13 +163,7 @@ class ClientWindow(GTKClientWindowBase, Gtk.Window):
         for k,v in hints.items():
             k = bytestostr(k)
             if k in INT_FIELDS:
-                if k.find("width")>=0:
-                    v = self._client.sx(v)
-                elif k.find("height")>=0:
-                    v = self._client.sy(v)
-                elif k.find("size")>=0:
-                    v = self._client.sp(v)
-                setattr(geom, k, int(v))
+                setattr(geom, k, v)
                 mask |= int(name_to_hint.get(k, 0))
             elif k in ASPECT_FIELDS:
                 field = ASPECT_FIELDS.get(k)
@@ -174,8 +179,12 @@ class ClientWindow(GTKClientWindowBase, Gtk.Window):
 
     def do_draw(self, context):
         paintlog("do_draw(%s)", context)
-        if self.get_mapped() and self._backing:
+        backing = self._backing
+        if self.get_mapped() and backing:
+            self.paint_backing_offset_border(backing, context)
+            self.clip_to_backing(backing, context)
             self._backing.cairo_draw(context)
+        self.cairo_paint_border(context, None)
 
 
 GObject.type_register(ClientWindow)
