@@ -1,26 +1,26 @@
 # This file is part of Xpra.
 # Copyright (C) 2010 Nathaniel Smith <njs@pobox.com>
-# Copyright (C) 2011-2017 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2011-2018 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 # A tray implemented using gtk.StatusIcon
 
 import os
-from xpra.os_util import WIN32, OSX, POSIX
+from xpra.os_util import WIN32, OSX, POSIX, PYTHON3, monotonic_time
 from xpra.util import envbool
-from xpra.gtk_common.gobject_compat import import_gtk, import_gdk, is_gtk3
+from xpra.gtk_common.gobject_compat import import_gtk, import_gdk
 gtk = import_gtk()
 gdk = import_gdk()
 
 from xpra.client.tray_base import TrayBase, log
-from xpra.gtk_common.gtk_util import get_icon_from_file, get_pixbuf_from_data, INTERP_HYPER
+from xpra.gtk_common.gtk_util import get_icon_from_file, get_pixbuf_from_data, get_default_root_window, \
+    INTERP_HYPER, SHIFT_MASK, ORIENTATION_VERTICAL, ORIENTATION_HORIZONTAL
 
-ORIENTATION = {}
-if not is_gtk3():
-    #where was this moved to??
-    ORIENTATION[gtk.ORIENTATION_HORIZONTAL] = "HORIZONTAL"
-    ORIENTATION[gtk.ORIENTATION_VERTICAL]   = "VERTICAL"
+ORIENTATION = {
+    ORIENTATION_HORIZONTAL  : "HORIZONTAL",
+    ORIENTATION_VERTICAL    : "VERTICAL",
+    }
 
 GUESS_GEOMETRY = WIN32 or OSX
 GUESS_GEOMETRY = envbool("XPRA_GUESS_ICON_GEOMETRY", GUESS_GEOMETRY)
@@ -44,31 +44,40 @@ class GTKStatusIconTray(TrayBase):
     def may_guess(self):
         log("may_guess() GUESS_GEOMETRY=%s, current guess=%s", GUESS_GEOMETRY, self.geometry_guess)
         if GUESS_GEOMETRY:
-            x, y = gdk.get_default_root_window().get_pointer()[:2]
+            x, y = get_default_root_window().get_pointer()[-3:-1]
             w, h = self.get_size()
             self.recalculate_geometry(x, y, w, h)
 
-    def activate_menu(self, widget, *args):
-        log("activate_menu(%s, %s)", widget, args)
-        self.may_guess()
-        if self.click_cb:
-            self.click_cb(1, 1)
-            self.click_cb(1, 0)
+    def activate_menu(self, widget):
+        modifiers_mask = get_default_root_window().get_pointer()[-1]
+        log("activate_menu(%s) modifiers_mask=%s", widget, modifiers_mask)
+        if (modifiers_mask & SHIFT_MASK) ^ OSX:
+            self.handle_click(2)
+        else:
+            self.handle_click(1)
 
     def popup_menu(self, widget, button, time, *args):
-        log("popup_menu(%s, %s, %s, %s)", widget, button, time, args)
+        modifiers_mask = get_default_root_window().get_pointer()[-1]
+        log("popup_menu(%s, %s, %s, %s) modifiers_mask=%s", widget, button, time, args, modifiers_mask)
+        if (modifiers_mask & SHIFT_MASK) ^ OSX:
+            self.handle_click(1)
+        else:
+            self.handle_click(2)
+
+    def handle_click(self, button, time=0):
+        log("handle_click(%i, %i)", button, time)
         self.may_guess()
         if self.click_cb:
             self.click_cb(button, 1, time)
             self.click_cb(button, 0, time)
 
 
-    def hide(self, *args):
+    def hide(self, *_args):
         log("%s.set_visible(False)", self.tray_widget)
         if self.tray_widget:
             self.tray_widget.set_visible(False)
 
-    def show(self, *args):
+    def show(self, *_args):
         log("%s.set_visible(True)", self.tray_widget)
         if self.tray_widget:
             self.tray_widget.set_visible(True)
@@ -104,13 +113,17 @@ class GTKStatusIconTray(TrayBase):
             ag = self.tray_widget.get_geometry()
             log("GTKStatusIconTray.get_geometry() %s.get_geometry()=%s", self.tray_widget, ag)
         if ag is None:
+            if not self.geometry_guess:
+                self.may_guess()
             #probably win32 or OSX, gnome-shell or KDE5..
-            self.may_guess()
             log("GTKStatusIconTray.get_geometry() no geometry value available, returning guess: %s", self.geometry_guess)
             return self.geometry_guess
         #gtk3 adds an extra argument.. at the beginning!
         _, geom, _ = ag[-3:]
         x, y, w, h = geom.x, geom.y, geom.width, geom.height
+        log("GTKStatusIconTray.get_geometry() geometry area rectangle=%s", (x, y, w, h))
+        if x==0 and y==0 and w==0 and h==0 and self.geometry_guess:
+            return self.geometry_guess
         if x==0 and y==0 and w==200 and h==200:
             #this isn't right, take a better guess, at least for the size:
             w = 24
@@ -131,7 +144,7 @@ class GTKStatusIconTray(TrayBase):
             self.tray_widget.set_blinking(on)
 
 
-    def set_icon_from_data(self, pixels, has_alpha, w, h, rowstride, options={}):
+    def set_icon_from_data(self, pixels, has_alpha, w, h, rowstride, _options={}):
         tray_icon = get_pixbuf_from_data(pixels, has_alpha, w, h, rowstride)
         self.set_icon_from_pixbuf(tray_icon)
 
@@ -142,16 +155,18 @@ class GTKStatusIconTray(TrayBase):
     def set_icon_from_pixbuf(self, tray_icon):
         if not tray_icon or not self.tray_widget:
             return
-        tw, th = self.get_geometry()[2:]
+        tw, th = self.get_geometry()[2:4]
+        if (tw<=2 or th<=2) or (tw==200 and th==200):
+            log("bogus tray icon size: %ix%i", tw, th)
+            tw = th = 48
         w = tray_icon.get_width()
         h = tray_icon.get_height()
         log("set_icon_from_pixbuf(%s) geometry=%s, icon size=%s", tray_icon, self.get_geometry(), (w, h))
         if tw!=w or th!=h:
-            if tw!=th:
+            if tw!=th and not PYTHON3:
                 #paste the scaled icon in the middle of the rectangle:
                 minsize = min(tw, th)
-                new_icon = gtk.gdk.Pixbuf(gtk.gdk.COLORSPACE_RGB, True, 8, tw, th)
-                new_icon.fill(0)
+                new_icon = get_pixbuf_from_data(b"\0"*tw*th*4, True, tw, th, tw*4)
                 scaled_w, scaled_h = minsize, minsize
                 if tw==24 and th==64:
                     #special case for the gnome-shell dimensions - stretch height..
@@ -162,6 +177,7 @@ class GTKStatusIconTray(TrayBase):
             else:
                 tray_icon = tray_icon.scale_simple(tw, th, INTERP_HYPER)
         self.tray_widget.set_from_pixbuf(tray_icon)
+        self.icon_timestamp = monotonic_time()
 
 
 def main():
@@ -169,7 +185,7 @@ def main():
     from xpra.gtk_common.gobject_compat import import_glib
     glib = import_glib()
     log.enable_debug()
-    s = GTKStatusIconTray(None, None, "test", "xpra.png", None, None, None, gtk.main_quit)
+    s = GTKStatusIconTray(None, None, None, "test", "xpra.png", None, None, None, gtk.main_quit)
     glib.timeout_add(1000*2, s.set_blinking, True)
     glib.timeout_add(1000*5, s.set_blinking, False)
     glib.timeout_add(1000*30, gtk.main_quit)
