@@ -7,9 +7,8 @@
 import os
 import struct
 import re
-import binascii
 
-from xpra.gtk_common.gobject_compat import import_gobject, import_gtk, import_gdk, import_glib
+from xpra.gtk_common.gobject_compat import import_gobject, import_gtk, import_gdk, import_glib, is_gtk3
 gobject = import_gobject()
 glib = import_glib()
 gtk = import_gtk()
@@ -19,11 +18,11 @@ from xpra.log import Logger
 log = Logger("clipboard")
 
 from xpra.gtk_common.gobject_util import no_arg_signal, SIGNAL_RUN_LAST
-from xpra.gtk_common.gtk_util import GetClipboard, PROPERTY_CHANGE_MASK
+from xpra.gtk_common.gtk_util import GetClipboard, selection_owner_set, selection_add_target, selectiondata_get_selection, selectiondata_get_target, selectiondata_get_data, selectiondata_get_data_type, selectiondata_get_format, selectiondata_set, clipboard_request_contents, PROPERTY_CHANGE_MASK
 from xpra.gtk_common.nested_main import NestedMainLoop
 from xpra.net.compression import Compressible
-from xpra.os_util import WIN32, POSIX, monotonic_time
-from xpra.util import csv, envint, envbool, repr_ellipsized
+from xpra.os_util import WIN32, POSIX, monotonic_time, strtobytes, bytestostr, hexstr, get_hex_uuid
+from xpra.util import csv, envint, envbool, repr_ellipsized, typedict
 from xpra.platform.features import CLIPBOARD_GREEDY
 
 
@@ -31,18 +30,21 @@ MIN_CLIPBOARD_COMPRESSION_SIZE = 512
 MAX_CLIPBOARD_PACKET_SIZE = 4*1024*1024
 
 from xpra.platform.features import CLIPBOARDS as PLATFORM_CLIPBOARDS
-ALL_CLIPBOARDS = PLATFORM_CLIPBOARDS
+ALL_CLIPBOARDS = [strtobytes(x) for x in PLATFORM_CLIPBOARDS]
 CLIPBOARDS = PLATFORM_CLIPBOARDS
 CLIPBOARDS_ENV = os.environ.get("XPRA_CLIPBOARDS")
 if CLIPBOARDS_ENV is not None:
     CLIPBOARDS = CLIPBOARDS_ENV.split(",")
-    CLIPBOARDS = [x.upper().strip() for x in CLIPBOARDS]
-
+    CLIPBOARDS = [strtobytes(x).upper().strip() for x in CLIPBOARDS]
+del CLIPBOARDS_ENV
 
 TEST_DROP_CLIPBOARD_REQUESTS = envint("XPRA_TEST_DROP_CLIPBOARD")
 STORE_ON_EXIT = envbool("XPRA_CLIPBOARD_STORE_ON_EXIT", True)
 DELAY_SEND_TOKEN = envint("XPRA_DELAY_SEND_TOKEN", 100)
 
+LOOP_DISABLE = envbool("XPRA_CLIPBOARD_LOOP_DISABLE", True)
+LOOP_PREFIX = os.environ.get("XPRA_CLIPBOARD_LOOP_PREFIX", "Xpra-Clipboard-Loop-Detection:")
+
 _discard_target_strs_ = os.environ.get("XPRA_DISCARD_TARGETS")
 if _discard_target_strs_ is not None:
     DISCARD_TARGETS = _discard_target_strs_.split(",")
@@ -64,6 +66,11 @@ TRANSLATED_TARGETS = {
     "application/x-moz-nativehtml" : "UTF8_STRING"
     }
 
+sizeof_long = struct.calcsize('@L')
+assert sizeof_long in (4, 8), "struct.calcsize('@L')=%s" % sizeof_long
+sizeof_short = struct.calcsize('=H')
+assert sizeof_short == 2, "struct.calcsize('=H')=%s" % sizeof_short
+
 
 MAX_NESTING = 20
 
@@ -78,13 +85,13 @@ def nesting_check():
 
 
 #may get overriden
-def nosanitize_gtkselectiondata(selectiondata):
+def nosanitize_gtkselectiondata(_selectiondata):
     return False
 sanitize_gtkselectiondata = nosanitize_gtkselectiondata
 
 
 def must_discard(target):
-    return any(x for x in DISCARD_TARGETS if x.match(target))
+    return any(x for x in DISCARD_TARGETS if x.match(bytestostr(target)))
 
 def _filter_targets(targets):
     f = [target for target in targets if not must_discard(target)]
@@ -92,27 +99,51 @@ def _filter_targets(targets):
     return f
 
 
+def set_string(clipboard, thestring):
+    if is_gtk3():
+        #no other way?
+        clipboard.set_text(thestring, len(thestring))
+        return
+    value = [thestring]
+    def get_func(clipboard, selection, info, targets):
+        log("get_func%s value=%s", (clipboard, selection, info, targets), value[0])
+        s = value[0]
+        if s:
+            selection.set("STRING", 8, s)
+        else:
+            clipboard.clear()
+    def clear_func(*args):
+        log("clear_func%s value=%s", args, value[0])
+        value[0] = ""
+    clipboard.set_with_data([("STRING", 0, 0)], get_func, clear_func)
+
+
 class ClipboardProtocolHelperBase(object):
     def __init__(self, send_packet_cb, progress_cb=None, **kwargs):
+        d = typedict(kwargs)
         self.send = send_packet_cb
         self.progress_cb = progress_cb
-        self.can_send = kwargs.get("can-send", True)
-        self.can_receive = kwargs.get("can-receive", True)
+        self.can_send = d.boolget("can-send", True)
+        self.can_receive = d.boolget("can-receive", True)
         self.max_clipboard_packet_size = MAX_CLIPBOARD_PACKET_SIZE
+        self.disabled_by_loop = []
         self.filter_res = []
-        filter_res = kwargs.get("filters")
+        filter_res = d.strlistget("filters")
         if filter_res:
             for x in filter_res:
                 try:
                     self.filter_res.append(re.compile(x))
-                except:
-                    log.error("invalid regular expression '%s' in clipboard filter")
+                except Exception as e:
+                    log.error("Error: invalid clipboard filter regular expression")
+                    log.error(" '%s': %s", x, e)
         self._clipboard_request_counter = 0
         self._clipboard_outstanding_requests = {}
         self._want_targets = False
         self.init_packet_handlers()
-        self.init_proxies(kwargs.get("clipboards.local", CLIPBOARDS))
-        self.remote_clipboards = kwargs.get("clipboards.remote", CLIPBOARDS)
+        self.init_proxies(d.strlistget("clipboards.local", CLIPBOARDS))
+        remote_loop_uuids = d.dictget("remote-loop-uuids", {})
+        self.verify_remote_loop_uuids(remote_loop_uuids)
+        self.remote_clipboards = d.strlistget("clipboards.remote", CLIPBOARDS)
 
     def __repr__(self):
         return "ClipboardProtocolHelperBase"
@@ -123,7 +154,7 @@ class ClipboardProtocolHelperBase(object):
                 "max_size"  : self.max_clipboard_packet_size,
                 "filters"   : [x.pattern for x in self.filter_res],
                 "requests"  : self._clipboard_request_counter,
-                "pending"   : self._clipboard_outstanding_requests.keys(),
+                "pending"   : tuple(self._clipboard_outstanding_requests.keys()),
                 "can-send"      : self.can_send,
                 "can-receive"   : self.can_receive,
                 "want_targets"  : self._want_targets,
@@ -141,6 +172,52 @@ class ClipboardProtocolHelperBase(object):
             x.cleanup()
         self._clipboard_proxies = {}
 
+
+    def client_reset(self):
+        #if the client disconnects,
+        #we can re-enable the clipboards it had problems with:
+        l = self.disabled_by_loop
+        self.disabled_by_loop = []
+        for x in l:
+            proxy = self._clipboard_proxies.get(x)
+            proxy.set_enabled(True)
+
+
+    def get_loop_uuids(self):
+        uuids = {}
+        for proxy in self._clipboard_proxies.values():
+            uuids[proxy._selection] = proxy._loop_uuid
+        log("get_loop_uuids()=%s", uuids)
+        return uuids
+
+    def verify_remote_loop_uuids(self, uuids):
+        log("verify_remote_loop_uuids(%s)", uuids)
+        if not uuids:
+            return
+        for proxy in self._clipboard_proxies.values():
+            proxy._clipboard.request_text(self._verify_remote_loop_uuids, (proxy, uuids))
+
+    def _verify_remote_loop_uuids(self, clipboard, value, user_data):
+        log("_verify_remote_loop_uuids(%s)", (clipboard, value, user_data))
+        proxy, uuids = user_data
+        if value:
+            for selection, rvalue in uuids.items():
+                log("%s=%s", proxy._selection, value)
+                if rvalue==proxy._loop_uuid:
+                    set_string(clipboard, "")
+                if rvalue and value==rvalue:
+                    set_string(clipboard, "")
+                    if selection==proxy._selection:
+                        log.warn("Warning: loop detected for %s clipboard", selection)
+                    else:
+                        log.warn("Warning: loop detected")
+                        log.warn(" local %s clipboard matches remote %s clipboard", proxy._selection, selection)
+                    if LOOP_DISABLE:
+                        log.warn(" synchronization has been disabled")
+                        proxy._enabled = False
+                        if selection not in self.disabled_by_loop:
+                            self.disabled_by_loop.append(selection)
+
     def set_direction(self, can_send, can_receive):
         self.can_send = can_send
         self.can_receive = can_receive
@@ -153,7 +230,7 @@ class ClipboardProtocolHelperBase(object):
         #(ie: OSX and win32 only use "CLIPBOARD" by default, and not "PRIMARY" or "SECONDARY")
         log("enabling selections: %s", csv(selections))
         for selection, proxy in self._clipboard_proxies.items():
-            proxy.set_enabled(selection in selections)
+            proxy.set_enabled(strtobytes(selection) in selections)
 
     def set_greedy_client(self, greedy):
         for proxy in self._clipboard_proxies.values():
@@ -165,12 +242,13 @@ class ClipboardProtocolHelperBase(object):
 
     def init_packet_handlers(self):
         self._packet_handlers = {
-            "clipboard-token":              self._process_clipboard_token,
-            "clipboard-request":            self._process_clipboard_request,
-            "clipboard-contents":           self._process_clipboard_contents,
-            "clipboard-contents-none":      self._process_clipboard_contents_none,
-            "clipboard-pending-requests":   self._process_clipboard_pending_requests,
-            "clipboard-enable-selections":  self._process_clipboard_enable_selections,
+            "clipboard-token"               : self._process_clipboard_token,
+            "clipboard-request"             : self._process_clipboard_request,
+            "clipboard-contents"            : self._process_clipboard_contents,
+            "clipboard-contents-none"       : self._process_clipboard_contents_none,
+            "clipboard-pending-requests"    : self._process_clipboard_pending_requests,
+            "clipboard-enable-selections"   : self._process_clipboard_enable_selections,
+            "clipboard-loop-uuids"          : self._process_clipboard_loop_uuids,
             }
 
     def make_proxy(self, clipboard):
@@ -187,6 +265,11 @@ class ClipboardProtocolHelperBase(object):
             self._clipboard_proxies[clipboard] = proxy
         log("%s.init_proxies : %s", self, self._clipboard_proxies)
 
+    def init_proxies_uuid(self):
+        for proxy in self._clipboard_proxies.values():
+            proxy.init_uuid()
+
+
     def local_to_remote(self, selection):
         #overriden in some subclasses (see: translated_clipboard)
         return  selection
@@ -201,7 +284,7 @@ class ClipboardProtocolHelperBase(object):
             self._send_clipboard_token_handler(proxy)
 
     def _process_clipboard_token(self, packet):
-        selection = packet[1]
+        selection = bytestostr(packet[1])
         name = self.remote_to_local(selection)
         proxy = self._clipboard_proxies.get(name)
         if proxy is None:
@@ -213,18 +296,21 @@ class ClipboardProtocolHelperBase(object):
             l("ignoring token for clipboard proxy name '%s' (no proxy)", name)
             return
         if not proxy.is_enabled():
-            log.warn("ignoring token for clipboard proxy name '%s' (disabled)", name)
+            l = log
+            if name not in self.disabled_by_loop:
+                l = log.warn
+            l("ignoring token for clipboard proxy name '%s' (disabled)", name)
             return
         log("process clipboard token selection=%s, local clipboard name=%s, proxy=%s", selection, name, proxy)
         targets = None
         target_data = None
-        if len(packet)>=3:
-            targets = packet[2]
-        if len(packet)>=8:
-            target_data = {}
-            target, dtype, dformat, wire_encoding, wire_data = packet[3:8]
-            raw_data = self._munge_wire_selection_to_raw(wire_encoding, dtype, dformat, wire_data)
-            target_data[target] = raw_data
+        if proxy._can_receive:
+            if len(packet)>=3:
+                targets = packet[2]
+            if len(packet)>=8:
+                target, dtype, dformat, wire_encoding, wire_data = packet[3:8]
+                raw_data = self._munge_wire_selection_to_raw(wire_encoding, dtype, dformat, wire_data)
+                target_data = {target : raw_data}
         #older versions always claimed the selection when the token is received:
         claim = True
         if len(packet)>=10:
@@ -236,7 +322,8 @@ class ClipboardProtocolHelperBase(object):
         synchronous_client = len(packet)>=11 and bool(packet[10])
         proxy.got_token(targets, target_data, claim, synchronous_client)
 
-    def _get_clipboard_from_remote_handler(self, proxy, selection, target):
+    def _get_clipboard_from_remote_handler(self, _proxy, selection, target):
+        assert self.can_receive
         if must_discard(target):
             log("invalid target '%s'", target)
             return None
@@ -256,6 +343,7 @@ class ClipboardProtocolHelperBase(object):
         return result
 
     def _clipboard_got_contents(self, request_id, dtype, dformat, data):
+        assert self.can_receive
         loop = self._clipboard_outstanding_requests.get(request_id)
         log("got clipboard contents for id=%s len=%s, loop=%s (type=%s, format=%s)",
               request_id, len(data or []), loop, dtype, dformat)
@@ -281,6 +369,9 @@ class ClipboardProtocolHelperBase(object):
             #first get the targets, then get the contents for targets we want to send (if any)
             def got_targets(dtype, dformat, targets):
                 log("got_targets for selection %s: %s, %s, %s", selection, dtype, dformat, targets)
+                if targets is None:
+                    send_token(rsel)
+                    return
                 #if there is a text target, send that too (just the first one that matches for now..)
                 send_now = [x for x in targets if x in TEXT_TARGETS]
                 def send_targets_only():
@@ -313,12 +404,13 @@ class ClipboardProtocolHelperBase(object):
         send_token(rsel)
 
     def _munge_raw_selection_to_wire(self, target, dtype, dformat, data):
+        log("_munge_raw_selection_to_wire%s", (target, dtype, dformat, data))
         # Some types just cannot be marshalled:
         if type in ("WINDOW", "PIXMAP", "BITMAP", "DRAWABLE",
                     "PIXEL", "COLORMAP"):
             log("skipping clipboard data of type: %s, format=%s, len(data)=%s", dtype, dformat, len(data or ""))
             return None, None
-        if target=="TARGETS" and dtype=="ATOM":
+        if target==b"TARGETS" and dtype==b"ATOM":
             #targets is special cased here
             #because we get the values in wire format already (not atoms)
             #thanks to the request_targets() function (required on win32)
@@ -332,37 +424,33 @@ class ClipboardProtocolHelperBase(object):
         log("_do_munge_raw_selection_to_wire(%s, %s, %s, %s:%s)", target, dtype, dformat, type(data), len(data or ""))
         if dformat == 32:
             #you should be using gdk_clipboard for atom support!
-            if dtype in ("ATOM", "ATOM_PAIR") and POSIX:
+            if dtype in (b"ATOM", b"ATOM_PAIR") and POSIX:
                 #we cannot handle gdk atoms here (but gdk_clipboard does)
                 return None, None
             #important note: on 64 bits, format=32 means 8 bytes, not 4
             #that's just the way it is...
-            sizeof_long = struct.calcsize('@L')
-            assert sizeof_long in (4, 8), "struct.calcsize('@L')=%s" % sizeof_long
             binfmt = "@" + "L" * (len(data) // sizeof_long)
             ints = struct.unpack(binfmt, data)
-            return "integers", ints
+            return b"integers", ints
         elif dformat == 16:
-            sizeof_short = struct.calcsize('=H')
-            assert sizeof_short == 2, "struct.calcsize('=H')=%s" % sizeof_short
             binfmt = "=" + "H" * (len(data) // sizeof_short)
             ints = struct.unpack(binfmt, data)
-            return "integers", ints
+            return b"integers", ints
         elif dformat == 8:
             for x in self.filter_res:
                 if x.match(data):
                     log.warn("clipboard buffer contains blacklisted pattern '%s' and has been dropped!", x.pattern)
                     return None, None
-            return "bytes", data
+            return b"bytes", data
         else:
             log.error("unhandled format %s for clipboard data type %s" % (dformat, dtype))
             return None, None
 
     def _munge_wire_selection_to_raw(self, encoding, dtype, dformat, data):
         log("wire selection to raw, encoding=%s, type=%s, format=%s, len(data)=%s", encoding, dtype, dformat, len(data or ""))
-        if encoding == "bytes":
+        if encoding == b"bytes":
             return data
-        elif encoding == "integers":
+        elif encoding == b"integers":
             if len(data or "")==0:
                 return ""
             if dformat == 32:
@@ -387,7 +475,7 @@ class ClipboardProtocolHelperBase(object):
             log("invalid target '%s'", target)
             no_contents()
             return
-        name = self.remote_to_local(selection)
+        name = self.remote_to_local(bytestostr(selection))
         log("process clipboard request, request_id=%s, selection=%s, local name=%s, target=%s", request_id, selection, name, target)
         proxy = self._clipboard_proxies.get(name)
         if proxy is None:
@@ -396,7 +484,14 @@ class ClipboardProtocolHelperBase(object):
             no_contents()
             return
         if not proxy.is_enabled():
-            log.warn("Warning: ignoring clipboard request for '%s' (disabled)", name)
+            l = log
+            if selection not in self.disabled_by_loop:
+                l = log.warn
+            l("Warning: ignoring clipboard request for '%s' (disabled)", name)
+            no_contents()
+            return
+        if not proxy._can_send:
+            log("request for %s but sending is disabled, sending 'none' back", name)
             no_contents()
             return
         if TEST_DROP_CLIPBOARD_REQUESTS>0 and (request_id % TEST_DROP_CLIPBOARD_REQUESTS)==0:
@@ -404,8 +499,8 @@ class ClipboardProtocolHelperBase(object):
             return
         def got_contents(dtype, dformat, data):
             log("got_contents(%s, %s, %s:%s) data=0x%s..",
-                  dtype, dformat, type(data), len(data or ""), binascii.hexlify(str(data)[:200]))
-            if dtype is None or data is None:
+                  dtype, dformat, type(data), len(data or ""), hexstr((data or "")[:200]))
+            if dtype is None or data is None or (dformat==0 and data==b""):
                 no_contents()
                 return
             munged = self._munge_raw_selection_to_wire(target, dtype, dformat, data)
@@ -450,11 +545,19 @@ class ClipboardProtocolHelperBase(object):
         selections = packet[1]
         self.enable_selections(selections)
 
+    def _process_clipboard_loop_uuids(self, packet):
+        loop_uuids = packet[1]
+        self.verify_remote_loop_uuids(loop_uuids)
+
 
     def process_clipboard_packet(self, packet):
-        packet_type = packet[0]
-        log("process clipboard packet type=%s", packet_type)
-        self._packet_handlers[packet_type](packet)
+        packet_type = bytestostr(packet[0])
+        handler = self._packet_handlers.get(packet_type)
+        log("process clipboard handler(%s)=%s", packet_type, handler)
+        if handler:
+            self._packet_handlers[packet_type](packet)
+        else:
+            log.warn("Warning: no clipboard packet handler for '%s'", packet_type)
 
 
 class DefaultClipboardProtocolHelper(ClipboardProtocolHelperBase):
@@ -483,6 +586,9 @@ class ClipboardProxy(gtk.Invisible):
         self._clipboard = GetClipboard(selection)
         self._enabled = True
         self._have_token = False
+        #enabled later during setup
+        self._can_send = False
+        self._can_receive = False
         #this workaround is only needed on win32 AFAIK:
         self._strip_nullbyte = WIN32
         #clients that need a new token for every owner-change: (ie: win32 and osx)
@@ -491,7 +597,7 @@ class ClipboardProxy(gtk.Invisible):
         #semaphore to block the sending of the token when we change the owner ourselves:
         self._block_owner_change = False
         self._last_emit_token = 0
-        self._emit_token_timer = 0
+        self._emit_token_timer = None
         #counters for info:
         self._selection_request_events = 0
         self._selection_get_events = 0
@@ -508,8 +614,14 @@ class ClipboardProxy(gtk.Invisible):
         except ImportError:
             self.prop_get = None
 
+        self._loop_uuid = ""
         self._clipboard.connect("owner-change", self.do_owner_changed)
 
+    def init_uuid(self):
+        self._loop_uuid = LOOP_PREFIX+get_hex_uuid()
+        log("init_uuid() %s uuid=%s", self._selection, self._loop_uuid)
+        set_string(self._clipboard, self._loop_uuid)
+
     def set_direction(self, can_send, can_receive):
         self._can_send = can_send
         self._can_receive = can_receive
@@ -521,6 +633,7 @@ class ClipboardProxy(gtk.Invisible):
                 "greedy_client"         : self._greedy_client,
                 "blocked_owner_change"  : self._block_owner_change,
                 "last-targets"          : self._last_targets,
+                "loop-uuid"             : self._loop_uuid,
                 "event"         : {
                                    "selection_request"     : self._selection_request_events,
                                    "selection_get"         : self._selection_get_events,
@@ -554,13 +667,13 @@ class ClipboardProxy(gtk.Invisible):
     def __repr__(self):
         return  "ClipboardProxy(%s)" % self._selection
 
-    def do_owner_changed(self, *args):
+    def do_owner_changed(self, *_args):
         #an application on our side owns the clipboard selection
         #(they are ready to provide something via the clipboard)
-        log("clipboard: %s owner_changed, enabled=%s, can-send=%s, can-receive=%s, have_token=%s, greedy_client=%s, block_owner_change=%s", self._selection, self._enabled, self._can_send, self._can_receive, self._have_token, self._greedy_client, self._block_owner_change)
+        log("clipboard: %s owner_changed, enabled=%s, can-send=%s, can-receive=%s, have_token=%s, greedy_client=%s, block_owner_change=%s", bytestostr(self._selection), self._enabled, self._can_send, self._can_receive, self._have_token, self._greedy_client, self._block_owner_change)
         if not self._enabled or self._block_owner_change:
             return
-        if self._have_token or self._greedy_client:
+        if self._have_token or (self._greedy_client and self._can_send):
             if self._have_token or DELAY_SEND_TOKEN<0:
                 #token ownership will change or told not to wait
                 glib.idle_add(self.emit_token)
@@ -633,7 +746,7 @@ class ClipboardProxy(gtk.Invisible):
         # assert that we can handle the requested sort of target. Fortunately,
         # Xpra never needs to request the clipboard when it owns it, so that's
         # okay.
-        assert str(event.selection) == self._selection
+        assert str(event.selection) == self._selection, "selection does not match: expected %s but got %s" % (event.selection, self._selection)
         target = str(event.target)
         if target == "TIMESTAMP":
             pass
@@ -647,10 +760,10 @@ class ClipboardProxy(gtk.Invisible):
             if atoms:
                 targets = atoms[::2]
                 for t in targets:
-                    self.selection_add_target(self._selection, t, 0)
+                    selection_add_target(self, self._selection, t, 0)
         else:
             log("target for %s: %r", self._selection, target)
-            self.selection_add_target(self._selection, target, 0)
+            selection_add_target(self, self._selection, target, 0)
         log("do_selection_request_event(%s) target=%s, selection=%s", event, target, self._selection)
         gtk.Invisible.do_selection_request_event(self, event)
 
@@ -661,14 +774,15 @@ class ClipboardProxy(gtk.Invisible):
         # In practice, send a call across the wire, then block in a recursive
         # main loop.
         def nodata():
-            selection_data.set("STRING", 8, "")
+            selectiondata_set(selection_data, "STRING", 8, "")
         if not self._enabled or not self._can_receive:
             nodata()
             return
-        log("do_selection_get(%s, %s, %s) selection=%s", selection_data, info, time, selection_data.selection)
+        selection = selectiondata_get_selection(selection_data)
+        target = selectiondata_get_target(selection_data)
+        log("do_selection_get(%s, %s, %s) selection=%s", selection_data, info, time, selection)
         self._selection_get_events += 1
-        assert self._selection == str(selection_data.selection)
-        target = str(selection_data.target)
+        assert str(selection) == self._selection, "selection does not match: expected %s but got %s" % (selection, self._selection)
         self._request_contents_events += 1
         result = self.emit("get-clipboard-from-remote", self._selection, target)
         if result is None or result["type"] is None:
@@ -682,7 +796,12 @@ class ClipboardProxy(gtk.Invisible):
               selection_data, info, time, dtype, dformat, type(data), len(data or ""))
         boc = self._block_owner_change
         self._block_owner_change = True
-        selection_data.set(dtype, dformat, data)
+        if is_gtk3() and dtype in (b"UTF8_STRING", b"STRING") and dformat==8:
+            #GTK3 workaround: can only use set_text and only on the clipboard?
+            s = bytestostr(data)
+            self._clipboard.set_text(s, len(s))
+        else:
+            selectiondata_set(selection_data, dtype, dformat, data)
         if boc is False:
             glib.idle_add(self.remove_block)
 
@@ -718,14 +837,14 @@ class ClipboardProxy(gtk.Invisible):
         if (CLIPBOARD_GREEDY or synchronous_client) and self._can_receive:
             if targets:
                 for target in targets:
-                    self.selection_add_target(self._selection, target, 0)
-                self.selection_owner_set(self._selection)
+                    selection_add_target(self, self._selection, target, 0)
+                selection_owner_set(self, self._selection)
             if target_data:
                 for text_target in TEXT_TARGETS:
                     if text_target in target_data:
                         text_data = target_data.get(text_target)
                         log("clipboard %s set to '%s'", self._selection, repr_ellipsized(text_data))
-                        self._clipboard.set_text(text_data, len=len(text_data))
+                        set_string(self._clipboard, text_data)
         if not claim:
             log("token packet without claim, not setting the token flag")
             #the other end is just telling us to send the token again next time something changes,
@@ -740,13 +859,13 @@ class ClipboardProxy(gtk.Invisible):
                 glib.idle_add(self.remove_block)
             self.claim()
 
-    def remove_block(self, *args):
+    def remove_block(self, *_args):
         log("remove_block: %s", self._selection)
         self._block_owner_change = False
 
     def claim(self):
         log("claim() selection=%s, enabled=%s", self._selection, self._enabled)
-        if self._enabled and not self.selection_owner_set(self._selection):
+        if self._enabled and not selection_owner_set(self, self._selection):
             # I don't know how this can actually fail, given that we pass
             # CurrentTime, but just in case:
             log.warn("Failed to acquire local clipboard %s; "
@@ -772,26 +891,36 @@ class ClipboardProxy(gtk.Invisible):
             #handle TARGETS using "request_targets"
             def got_targets(c, targets, *args):
                 log("got_targets(%s, %s, %s)", c, targets, args)
+                if is_gtk3():
+                    targets = [x.name() for x in targets]
                 cb("ATOM", 32, targets)
                 self._last_targets = targets or ()
             self._clipboard.request_targets(got_targets)
             return
-        def unpack(clipboard, selection_data, user_data):
+        def unpack(clipboard, selection_data, _user_data=None):
             log("unpack %s: %s", clipboard, type(selection_data))
             global sanitize_gtkselectiondata
-            if sanitize_gtkselectiondata(selection_data):
+            if selection_data and sanitize_gtkselectiondata(selection_data):
+                selection_data = None
+            if selection_data is None:
                 cb(None, None, None)
                 return
             log("unpack: %s", selection_data)
-            data = selection_data.data
-            log("unpack(..) type=%s, format=%s, data=%s:%s", selection_data.type, selection_data.format,
-                        type(data), len(data or ""))
-            if self._strip_nullbyte and selection_data.type in ("UTF8_STRING", "STRING") and selection_data.format==8:
-                #we may have to strip the nullbyte:
-                if data and data[-1]=='\0':
-                    log("stripping end of string null byte")
-                    data = data[:-1]
-            cb(str(selection_data.type), selection_data.format, data)
+            data = selectiondata_get_data(selection_data)
+            dtype = selectiondata_get_data_type(selection_data)
+            dformat = selectiondata_get_format(selection_data)
+            log("unpack(..) type=%s, format=%s, data=%s:%s", dtype, dformat, type(data), len(data or ""))
+            isstring = dtype in (b"UTF8_STRING", b"STRING") and dformat==8
+            if isstring:
+                if self._strip_nullbyte:
+                    #we may have to strip the nullbyte:
+                    if data and data[-1]=='\0':
+                        log("stripping end of string null byte")
+                        data = data[:-1]
+                if data and data==self._loop_uuid:
+                    log("not sending loop uuid value '%s', returning an empty string instead", data)
+                    data= ""
+            cb(str(dtype), dformat, data)
         #some applications (ie: firefox, thunderbird) can request invalid targets,
         #when that happens, translate it to something the application can handle (if any)
         translated_target = TRANSLATED_TARGETS.get(target)
@@ -799,6 +928,6 @@ class ClipboardProxy(gtk.Invisible):
             (translated_target in self._last_targets) and (not must_discard(translated_target)):
             log("invalid target %s, replaced with %s", target, translated_target)
             target = translated_target
-        self._clipboard.request_contents(target, unpack)
+        clipboard_request_contents(self._clipboard, target, unpack)
 
 gobject.type_register(ClipboardProxy)
