@@ -37,6 +37,12 @@ def device_info(d):
         return "None"
     return "%s @ %s" % (d.name(), d.pci_bus_id())
 
+def device_name(d):
+    if not d:
+        return "None"
+    return d.name()
+
+
 def get_pycuda_version():
     return pycuda.VERSION
 
@@ -67,6 +73,10 @@ DEVICE_INFO = {}
 def get_device_info(i):
     global DEVICE_INFO
     return DEVICE_INFO.get(i, None)
+DEVICE_NAME = {}
+def get_device_name(i):
+    global DEVICE_NAME
+    return DEVICE_NAME.get(i, None)
 
 
 PREFS = None
@@ -115,7 +125,7 @@ def get_pref(name):
 
 DEVICES = None
 def init_all_devices():
-    global DEVICES, DEVICE_INFO
+    global DEVICES, DEVICE_INFO, DEVICE_NAME
     if DEVICES is not None:
         return DEVICES
     log.info("CUDA initialization (this may take a few seconds)")
@@ -150,6 +160,7 @@ def init_all_devices():
                     continue
             log(" + testing device %s: %s", i, devinfo)
             DEVICE_INFO[i] = devinfo
+            DEVICE_NAME[i] = device_name(device)
             host_mem = device.get_attribute(da.CAN_MAP_HOST_MEMORY)
             if not host_mem:
                 log.warn("skipping device %s (cannot map host memory)", devinfo)
@@ -216,7 +227,7 @@ def select_device(preferred_device_id=-1, preferred_device_name=None, min_comput
     ok_devices = [device_id for device_id in devices if DEVICE_STATE.get(device_id, True) is True]
     nok_devices = [device_id for device_id in devices if DEVICE_STATE.get(device_id, True) is not True]
     for list_name, device_list in {"OK" : ok_devices, "failing" : nok_devices}.items():
-        selected_device_id = None
+        selected_device_id = -1
         selected_device = None
         log("will test %s device%s from %s list: %s", len(device_list), engs(device_list), list_name, device_list)
         for device_id in device_list:
@@ -334,9 +345,9 @@ def get_CUDA_function(device_id, function_name):
     global KERNELS
     data = KERNELS.get(function_name)
     if data is None:
-        from xpra.platform.paths import default_get_app_dir
+        from xpra.platform.paths import get_app_dir
         from xpra.os_util import load_binary_file
-        cubin_file = os.path.join(default_get_app_dir(), "cuda", "%s.fatbin" % function_name)
+        cubin_file = os.path.join(get_app_dir(), "cuda", "%s.fatbin" % function_name)
         log("get_CUDA_function(%s, %s) cubin file=%s", device_id, function_name, cubin_file)
         data = load_binary_file(cubin_file)
         if not data:
