@@ -27,22 +27,32 @@ def want_gtk3(v):
     assert (_is_gtk3 is None or _is_gtk3==v), "cannot set gtk3=%s, already set to %s" % (v, _is_gtk3)
     _is_gtk3 = v
 
+def gtk_version():
+    return 2+int(is_gtk3())
 
 def _try_import(import_method_gtk3, import_method_gtk2):
     global _is_gtk3
     if "gi" in sys.modules:
         _is_gtk3 = True
     if _is_gtk3 is False:
-        return  import_method_gtk2()
+        return import_method_gtk2()
+    def trygtk3():
+        #if the introspection data is missing,
+        #we can get a ValueError
+        #but this confuses a number of tools that try to load our code (ie: py2app)
+        try:
+            return import_method_gtk3()
+        except ValueError as e:
+            raise ImportError(e)
     if _is_gtk3 is True:
-        return  import_method_gtk3()
+        return trygtk3()
     #python3 sets _is_gtk3 early
     assert sys.version_info[0]<3
     try:
         imported = import_method_gtk2()
         _is_gtk3 = False
     except:
-        imported = import_method_gtk3()
+        imported = trygtk3()
         _is_gtk3 = True
     return imported
 
@@ -56,15 +66,10 @@ def try_import_GdkX11():
             import gi
             gi.require_version('GdkX11', '3.0')
             from gi.repository import GdkX11            #@UnresolvedImport @UnusedImport
+            return GdkX11
         except:
             pass
-
-
-def get_xid(window):
-    if is_gtk3():
-        return window.get_xid()
-    else:
-        return window.xid
+    return None
 
 
 def import_gobject2():
@@ -146,6 +151,15 @@ def import_pango3():
 def import_pango():
     return  _try_import(import_pango3, import_pango2)
 
+def import_pangocairo2():
+    import pangocairo
+    return pangocairo
+def import_pangocairo3():
+    from gi.repository import PangoCairo            #@UnresolvedImport
+    return PangoCairo
+def import_pangocairo():
+    return  _try_import(import_pangocairo3, import_pangocairo2)
+
 def import_cairo():
     #we cannot use cairocffi with the do_paint callbacks..
     #import cairocffi                            #@UnresolvedImport
@@ -153,3 +167,15 @@ def import_cairo():
     #cairo = cairocffi
     import cairo
     return cairo
+
+def import_gtkosx_application2():
+    import gtkosx_application                   #@UnresolvedImport
+    return gtkosx_application
+def import_gtkosx_application3():
+    import gi
+    gi.require_version('GtkosxApplication', '1.0')
+    from gi.repository import GtkosxApplication #@UnresolvedImport
+    return GtkosxApplication
+def import_gtkosx_application():
+    return _try_import(import_gtkosx_application3, import_gtkosx_application2)
+
