@@ -1,13 +1,13 @@
 # This file is part of Xpra.
-# Copyright (C) 2011-2017 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2011-2018 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 import os
 import ctypes
 from xpra.util import roundup
-from xpra.os_util import memoryview_to_bytes, WIN32, POSIX
-from xpra.simple_stats import to_std_unit
+from xpra.os_util import memoryview_to_bytes, shellsub, WIN32, POSIX
+from xpra.simple_stats import std_unit
 from xpra.log import Logger
 log = Logger("mmap")
 
@@ -29,12 +29,16 @@ def init_client_mmap(mmap_group=None, socket_filename=None, size=128*1024*1024,
     mmap_filename = filename
     mmap_temp_file = None
     delete = True
+    def validate_size(size):
+        assert size>=64*1024*1024, "mmap size is too small: %sB (minimum is 64MB)" % std_unit(size)
+        assert size<=4*1024*1024*1024, "mmap is too big: %sB (maximum is 4GB)" % std_unit(size)
     try:
         import mmap
         unit = max(4096, mmap.PAGESIZE)
         #add 8 bytes for the mmap area control header zone:
         mmap_size = roundup(size + 8, unit)
         if WIN32:
+            validate_size(mmap_size)
             if not filename:
                 from xpra.net.crypto import get_hex_uuid
                 filename = "xpra-%s" % get_hex_uuid()
@@ -49,10 +53,12 @@ def init_client_mmap(mmap_group=None, socket_filename=None, size=128*1024*1024,
                 if os.path.exists(filename):
                     fd = os.open(filename, os.O_EXCL | os.O_RDWR)
                     mmap_size = os.path.getsize(mmap_filename)
+                    validate_size(mmap_size)
                     #mmap_size = 4*1024*1024    #size restriction needed with ivshmem
                     delete = False
                     log.info("Using existing mmap file '%s': %sMB", mmap_filename, mmap_size//1024//1024)
                 else:
+                    validate_size(mmap_size)
                     import errno
                     flags = os.O_CREAT | os.O_EXCL | os.O_RDWR
                     try:
@@ -65,10 +71,21 @@ def init_client_mmap(mmap_group=None, socket_filename=None, size=128*1024*1024,
                             return rerr()
                         raise
             else:
+                validate_size(mmap_size)
                 import tempfile
-                mmap_dir = os.getenv("TMPDIR", "/tmp")
-                if not os.path.exists(mmap_dir):
-                    raise Exception("TMPDIR %s does not exist!" % mmap_dir)
+                from xpra.platform.paths import get_mmap_dir
+                mmap_dir = get_mmap_dir()
+                subs = os.environ.copy()
+                subs.update({
+                    "UID"               : os.getuid(),
+                    "GID"               : os.getgid(),
+                    "PID"               : os.getpid(),
+                    })
+                mmap_dir = shellsub(mmap_dir, subs)
+                if mmap_dir and not os.path.exists(mmap_dir):
+                    os.mkdir(mmap_dir, 0o700)
+                if not mmap_dir or not os.path.exists(mmap_dir):
+                    raise Exception("mmap directory %s does not exist!" % mmap_dir)
                 #create the mmap file, the mkstemp that is called via NamedTemporaryFile ensures
                 #that the file is readable and writable only by the creating user ID
                 try:
@@ -87,8 +104,6 @@ def init_client_mmap(mmap_group=None, socket_filename=None, size=128*1024*1024,
                 s = os.stat(socket_filename)
                 os.fchown(fd, -1, s.st_gid)
                 os.fchmod(fd, S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP)
-            assert mmap_size>=1024*1024, "mmap size is too small: %s (minimum is 1MB)" % to_std_unit(mmap_size)
-            assert mmap_size<=1024*1024*1024, "mmap is too big: %s (maximum is 1GB)" % to_std_unit(mmap_size)
             log("using mmap file %s, fd=%s, size=%s", mmap_filename, fd, mmap_size)
             os.lseek(fd, mmap_size-1, os.SEEK_SET)
             assert os.write(fd, b'\x00')
@@ -191,12 +206,12 @@ def mmap_read(mmap_area, *descr_data):
         data_start.value = offset+length
         return arraytype.from_buffer(mmap_area, offset)
     #re-construct the buffer from discontiguous chunks:
-    data = ""
+    data = []
     for offset, length in descr_data:
         mmap_area.seek(offset)
-        data += mmap_area.read(length)
+        data.append(mmap_area.read(length))
         data_start.value = offset+length
-    return data
+    return b"".join(data)
 
 
 def mmap_write(mmap_area, mmap_size, data):
@@ -254,7 +269,7 @@ def mmap_write(mmap_area, mmap_size, data):
         #[+++++++++**********E--------------]
         mmap_area.seek(end)
         mmap_area.write(memoryview_to_bytes(data))
-        data = [(end, l)]
+        chunks = [(end, l)]
         mmap_data_end.value = end+l
     else:
         """ data does not fit in first chunk alone """
@@ -264,7 +279,7 @@ def mmap_write(mmap_area, mmap_size, data):
             #[*******E----------S+++++++++-------]
             mmap_area.seek(8)
             mmap_area.write(memoryview_to_bytes(data))
-            data = [(8, l)]
+            chunks = [(8, l)]
             mmap_data_end.value = 8+l
         else:
             """ split in 2 chunks: wrap around the end of the mmap buffer """
@@ -275,7 +290,7 @@ def mmap_write(mmap_area, mmap_size, data):
             mmap_area.seek(8)
             mmap_area.write(memoryview_to_bytes(data[chunk:]))
             l2 = l-chunk
-            data = [(end, chunk), (8, l2)]
+            chunks = [(end, chunk), (8, l2)]
             mmap_data_end.value = 8+l2
     log("sending damage with mmap: %s", data)
-    return data, mmap_free_size
+    return chunks, mmap_free_size
