@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # This file is part of Xpra.
-# Copyright (C) 2011-2017 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2011-2018 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -9,25 +9,34 @@ import math
 import ctypes
 import struct
 import weakref
-import gtkosx_application           #@UnresolvedImport
 
 import objc                         #@UnresolvedImport
 import Quartz                       #@UnresolvedImport
 import Quartz.CoreGraphics as CG    #@UnresolvedImport
-from Quartz import CGWindowListCopyWindowInfo, kCGWindowListOptionOnScreenOnly, kCGNullWindowID, kCGWindowListOptionAll #@UnresolvedImport
+from Quartz import CGWindowListCopyWindowInfo, kCGDisplaySetModeFlag, kCGWindowListOptionOnScreenOnly, kCGNullWindowID, kCGWindowListOptionAll #@UnresolvedImport
+from Quartz.CoreGraphics import CGDisplayRegisterReconfigurationCallback, CGDisplayRemoveReconfigurationCallback #@UnresolvedImport
 from AppKit import NSAppleEventManager, NSScreen, NSObject, NSBeep   #@UnresolvedImport
 from AppKit import NSApp, NSApplication, NSWorkspace, NSWorkspaceActiveSpaceDidChangeNotification, NSWorkspaceWillSleepNotification, NSWorkspaceDidWakeNotification     #@UnresolvedImport
+from Foundation import NSUserNotification, NSUserNotificationCenter, NSUserNotificationDefaultSoundName #@UnresolvedImport
 
+from xpra.os_util import PYTHON2
 from xpra.util import envbool, envint, roundup
+from xpra.notifications.notifier_base import NotifierBase
+
 from xpra.log import Logger
 log = Logger("osx", "events")
 workspacelog = Logger("osx", "events", "workspace")
-scrolllog = Logger("osx", "events", "scroll")
+mouselog = Logger("osx", "events", "mouse")
+notifylog = Logger("osx", "notify")
 
 
 OSX_FOCUS_WORKAROUND = envint("XPRA_OSX_FOCUS_WORKAROUND", 2000)
 SLEEP_HANDLER = envbool("XPRA_OSX_SLEEP_HANDLER", True)
+OSX_WHEEL_MULTIPLIER = envint("XPRA_OSX_WHEEL_MULTIPLIER", 100)
+OSX_WHEEL_PRECISE_MULTIPLIER = envint("XPRA_OSX_WHEEL_PRECISE_MULTIPLIER", 1)
 WHEEL = envbool("XPRA_WHEEL", True)
+NATIVE_NOTIFIER = envbool("XPRA_OSX_NATIVE_NOTIFIER", False)
+SUBPROCESS_NOTIFIER = envbool("XPRA_OSX_SUBPROCESS_NOTIFIER", False)
 
 ALPHA = {
          CG.kCGImageAlphaNone                  : "AlphaNone",
@@ -47,7 +56,7 @@ except:
 
 
 exit_cb = None
-def quit_handler(*args):
+def quit_handler(*_args):
     global exit_cb
     if exit_cb:
         exit_cb()
@@ -65,6 +74,8 @@ macapp = None
 def get_OSXApplication():
     global macapp
     if macapp is None:
+        from xpra.gtk_common.gobject_compat import import_gtkosx_application
+        gtkosx_application = import_gtkosx_application()
         macapp = gtkosx_application.Application()
         macapp.connect("NSApplicationWillTerminate", quit_handler)
     return macapp
@@ -93,11 +104,85 @@ def do_ready():
         osxapp.ready()
 
 
-def get_native_tray_menu_helper_classes():
+class OSX_Notifier(NotifierBase):
+
+    def __init__(self, closed_cb=None, action_cb=None):
+        NotifierBase.__init__(self, closed_cb, action_cb)
+        self.notifications = {}
+        self.notification_center = NSUserNotificationCenter.defaultUserNotificationCenter()
+        assert self.notification_center
+
+    def show_notify(self, dbus_id, tray, nid, app_name, replaces_nid, app_icon, summary, body, actions, hints, expire_timeout, icon):
+        notification = NSUserNotification.alloc().init()
+        notification.setTitle_(summary)
+        notification.setInformativeText_(body)
+        notification.setIdentifier_("%s" % nid)
+        #enable sound:
+        notification.setSoundName_(NSUserNotificationDefaultSoundName)
+        notifylog("show_notify(..) nid=%s, %s(%s)", nid, self.notification_center.deliverNotification_, notification)
+        self.notifications[nid] = notification
+        self.notification_center.deliverNotification_(notification)
+
+    def close_notify(self, nid):
+        notification = self.notifications.get(nid)
+        notifylog("close_notify(..) notification[%i]=%s", nid, notification)
+        if notification:
+            self.notification_center.removeDeliveredNotification_(notification)
+
+    def cleanup(self):
+        NotifierBase.cleanup(self)
+        self.notification_center.removeAllDeliveredNotifications()
+
+
+class OSX_Subprocess_Notifier(NotifierBase):
+    def show_notify(self, dbus_id, tray, nid, app_name, replaces_nid, app_icon, summary, body, actions, hints, expire_timeout, icon):
+        from xpra.platform.darwin import osx_notifier
+        osx_notifier_file = osx_notifier.__file__
+        if osx_notifier_file.endswith("pyc"):
+            osx_notifier_file = osx_notifier_file[:-1]
+        import time
+        #osx_notifier_file = "/Users/osx/osx_notifier.py"
+        from xpra.platform.paths import get_app_dir
+        base = get_app_dir()
+        #python_bin = "/usr/bin/python"
+        python_bin = os.path.join(base, "Resources", "bin", "python")
+        cmd = [python_bin, osx_notifier_file, "%s-%s" % (int(time.time()), nid), summary, body]
+        from xpra.child_reaper import getChildReaper
+        import subprocess
+        env = os.environ.copy()
+        for x in ("DYLD_LIBRARY_PATH", "XDG_CONFIG_DIRS", "XDG_DATA_DIRS",
+                  "GTK_DATA_PREFIX", "GTK_EXE_PREFIX", "GTK_PATH",
+                  "GTK2_RC_FILES", "GTK_IM_MODULE_FILE", "GDK_PIXBUF_MODULE_FILE",
+                  "PANGO_RC_FILE", "PANGO_LIBDIR", "PANGO_SYSCONFDIR",
+                  "CHARSETALIASDIR",
+                  "GST_BUNDLE_CONTENTS", "PYTHON", "PYTHONHOME",
+                  "PYTHONPATH"):
+            if x in env:
+                del env[x]
+        notifylog("running %s with env=%s", cmd, env)
+        proc = subprocess.Popen(cmd, env=env)
+        proc.wait()
+        notifylog("returned %i", proc.poll())
+        getChildReaper().add_process(proc, "notifier-%s" % nid, cmd, True, True)
+
+    def close_notify(self, nid):
+        pass
+    
+
+def get_native_notifier_classes():
+    v = []
+    if NATIVE_NOTIFIER and NSUserNotificationCenter.defaultUserNotificationCenter():
+        v.append(OSX_Notifier)
+    if SUBPROCESS_NOTIFIER:
+        v.append(OSX_Subprocess_Notifier)
+    notifylog("get_native_notifier_classes()=%s", v)
+    return v
+
+def get_native_tray_menu_helper_class():
     if get_OSXApplication():
         from xpra.platform.darwin.osx_menu import getOSXMenuHelper
-        return [getOSXMenuHelper]
-    return []
+        return getOSXMenuHelper
+    return None
 
 def get_native_tray_classes():
     if get_OSXApplication():
@@ -105,7 +190,7 @@ def get_native_tray_classes():
         return [OSXTray]
     return []
 
-def system_bell(*args):
+def system_bell(*_args):
     NSBeep()
     return True
 
@@ -278,6 +363,7 @@ def _call_CG_conv(defs, argument):
                 v = fn(argument)
             except Exception as e:
                 log("function %s failed: %s", fn_name, e)
+                del e
                 continue
             if v is not None:
                 info[prop_name] = conv(v)
@@ -438,26 +524,28 @@ VIEW_TO_WINDOW = weakref.WeakValueDictionary()
 
 def add_window_hooks(window):
     window.connect("focus-in-event", window_focused)
-    global VIEW_TO_WINDOW
-    try:
-        gdkwin = window.get_window()
-        VIEW_TO_WINDOW[gdkwin.nsview] = window
-    except:
-        log("add_window_hooks(%s)", window, exc_info=True)
-        log.error("Error: failed to associate window %s with its nsview", window, exc_info=True)
+    if WHEEL and PYTHON2:
+        global VIEW_TO_WINDOW
+        try:
+            gdkwin = window.get_window()
+            VIEW_TO_WINDOW[gdkwin.nsview] = window
+        except:
+            log("add_window_hooks(%s)", window, exc_info=True)
+            log.error("Error: failed to associate window %s with its nsview", window, exc_info=True)
 
 def remove_window_hooks(window):
     try:
         del window_menus[window]
     except:
         pass
-    global VIEW_TO_WINDOW
-    #this should be redundant as we use weak references:
-    try:
-        gdkwin = window.get_window()
-        del VIEW_TO_WINDOW[gdkwin.nsview]
-    except:
-        pass
+    if WHEEL and PYTHON2:
+        global VIEW_TO_WINDOW
+        #this should be redundant as we use weak references:
+        try:
+            gdkwin = window.get_window()
+            del VIEW_TO_WINDOW[gdkwin.nsview]
+        except:
+            pass
 
 
 def _set_osx_window_menu(add, wid, window, menus, application_action_callback=None, window_action_callback=None):
@@ -521,8 +609,8 @@ def register_URL_handler(handler):
     log("register_URL_handler(%s)", handler)
     class GURLHandler(NSObject):
         def handleEvent_withReplyEvent_(self, event, reply_event):
-            log("GURLHandler.handleEvent")
-            url = event.descriptorForKeyword_(fourCharToInt('----')).stringValue()
+            log("GURLHandler.handleEvent(%s, %s)", event, reply_event)
+            url = event.descriptorForKeyword_(fourCharToInt(b'----')).stringValue()
             log("URL=%s", url)
             handler(url.encode())
 
@@ -536,7 +624,7 @@ def register_URL_handler(handler):
     manager = NSAppleEventManager.sharedAppleEventManager()
     manager.setEventHandler_andSelector_forEventClass_andEventID_(
         urlh, 'handleEvent:withReplyEvent:',
-        fourCharToInt('GURL'), fourCharToInt('GURL')
+        fourCharToInt(b'GURL'), fourCharToInt(b'GURL')
         )
 
 
@@ -545,44 +633,40 @@ class Delegate(NSObject):
         log("applicationDidFinishLaunching_(%s)", notification)
         if SLEEP_HANDLER:
             self.register_sleep_handlers()
-        if WHEEL:
+        if WHEEL and PYTHON2:
             from xpra.platform.darwin.gdk_bindings import init_quartz_filter, set_wheel_event_handler   #@UnresolvedImport
             set_wheel_event_handler(self.wheel_event_handler)
             init_quartz_filter()
 
     @objc.python_method
-    def wheel_event_handler(self, nsview, deltax, deltay):
+    def wheel_event_handler(self, nsview, deltax, deltay, precise):
         global VIEW_TO_WINDOW
         window = VIEW_TO_WINDOW.get(nsview)
-        scrolllog("wheel_event_handler%s window=%s", (nsview, deltax, deltay), window)
+        mouselog("wheel_event_handler(%#x, %.4f, %.4f, %s) window=%s", nsview, deltax, deltay, precise, window)
         if not window:
             return False    #not handled
-        client = window._client
-        wid = window._id
-        modifiers = client.get_current_modifiers()
-        buttons = []
-        pointer = client.get_mouse_position()
-        def send_button(button, distance):
-            v = math.sqrt(10.0*abs(distance))       #ie: 0.1 -> 1, -0.9 -> 3
-            scrolllog("send_button(%i, %f) steps=%i", button, distance, v)
-            for _ in range(int(v)):
-                window._client.send_button(wid, button, True, pointer, modifiers, buttons)
-                window._client.send_button(wid, button, False, pointer, modifiers, buttons)
-        #accumulate deltas:
-        dx = getattr(window, "deltax", 0)+deltax
-        dy = getattr(window, "deltay", 0)+deltay
-        if abs(dx)>0.1:
-            button = client.wheel_map.get(6+int(dx>0))            #RIGHT=7, LEFT=6
-            if button>0:
-                send_button(button, dx)
-                dx = 0
-        if abs(dy)>0.1:
-            button = client.wheel_map.get(5-int(dy>0))            #UP=4, DOWN=5
-            if button>0:
-                send_button(button, dy)
-                dy = 0
-        setattr(window, "deltax", dx)
-        setattr(window, "deltay", dy)
+        def normalize_precision(distance):
+            if distance==0:
+                return 0
+            if precise:
+                m = OSX_WHEEL_PRECISE_MULTIPLIER
+            else:
+                m = OSX_WHEEL_MULTIPLIER
+            v = abs(distance)/10.0*m
+            if v>1:
+                #cancel out some of the crazy fast scroll acceleration from macos:
+                v = math.sqrt(v)
+            if distance<0:
+                #restore sign:
+                v = -v
+            mouselog("normalize_precision(%.3f)=%.3f (multiplier=%i)", distance, v, m)
+            return v
+        dx = normalize_precision(deltax)
+        dy = normalize_precision(deltay)
+        if dx!=0 or dy!=0:
+            client = window._client
+            wid = window._id
+            client.wheel_event(wid, dx, dy)
         return True
 
     @objc.python_method
@@ -597,7 +681,7 @@ class Delegate(NSObject):
         add_observer(self.receiveWakeNotification_, NSWorkspaceDidWakeNotification)
         add_observer(self.receiveWorkspaceChangeNotification_, NSWorkspaceActiveSpaceDidChangeNotification)
 
-    @objc.signature('B@:#B')
+    @objc.signature(b'B@:#B')
     def applicationShouldHandleReopen_hasVisibleWindows_(self, ns_app, flag):
         log("applicationShouldHandleReopen_hasVisibleWindows%s", (ns_app, flag))
         self.delegate_cb("deiconify_callback")
@@ -645,26 +729,48 @@ class Delegate(NSObject):
                 log.error("Error in %s callback %s", name, callback, exc_info=True)
 
 
+def disable_focus_workaround():
+    NSApp.activateIgnoringOtherApps_(False)
+def enable_focus_workaround():
+    NSApp.activateIgnoringOtherApps_(True)
+
+
 class ClientExtras(object):
     def __init__(self, client, opts):
         if OSX_FOCUS_WORKAROUND:
-            def disable_focus_workaround(*args):
-                NSApp.activateIgnoringOtherApps_(False)
-            def enable_focus_workaround(*args):
-                NSApp.activateIgnoringOtherApps_(True)
+            def first_ui_received(*_args):
+                enable_focus_workaround()
                 client.timeout_add(OSX_FOCUS_WORKAROUND, disable_focus_workaround)
-            client.connect("first-ui-received", enable_focus_workaround)
+            client.connect("first-ui-received", first_ui_received)
         swap_keys = opts and opts.swap_keys
         log("ClientExtras.__init__(%s, %s) swap_keys=%s", client, opts, swap_keys)
         self.client = client
         self.event_loop_started = False
+        self.check_display_timer = 0
+        self.display_is_asleep = False
         if opts and client:
             log("setting swap_keys=%s using %s", swap_keys, client.keyboard_helper)
             if client.keyboard_helper and client.keyboard_helper.keyboard:
                 log("%s.swap_keys=%s", client.keyboard_helper.keyboard, swap_keys)
                 client.keyboard_helper.keyboard.swap_keys = swap_keys
+        if client:
+            self.check_display_timer = client.timeout_add(60*1000, self.check_display)
 
     def cleanup(self):
+        cdt = self.check_display_timer
+        client = self.client
+        if cdt and client:
+            client.source_remove(cdt)
+            self.check_display_timer = 0
+        try:
+            r = CGDisplayRemoveReconfigurationCallback(self.display_change, self)
+        except ValueError as e:
+            log("CGDisplayRemoveReconfigurationCallback: %s", e)
+            #if we exit from a signal, this may fail
+            r = 1
+        if r!=0:
+            #don't bother logging this as a warning since we are terminating anyway:
+            log("failed to unregister display reconfiguration callback")
         self.client = None
 
     def ready(self):
@@ -690,7 +796,36 @@ class ClientExtras(object):
             self.delegate.deiconify_callback = self.client.deiconify_windows
         self.shared_app.setDelegate_(self.delegate)
         log("setup_event_listener() the application delegate has been registered")
+        r = CGDisplayRegisterReconfigurationCallback(self.display_change, self)
+        if r!=0:
+            log.warn("Warning: failed to register display reconfiguration callback")
 
+    def display_change(self, display, flags, userinfo):
+        log("display_change%s", (display, flags, userinfo))
+        c = self.client
+        #The display mode has changed
+        #opengl windows may need to be re-created since the GPU may have changed:
+        if (flags & kCGDisplaySetModeFlag) and c and c.opengl_enabled:
+            c.reinit_windows()
+
+    def check_display(self):
+        log("check_display()")
+        try:
+            did = CG.CGMainDisplayID()
+            log("check_display() CGMainDisplayID()=%#x", did)
+            if did and self.client:
+                asleep = bool(CG.CGDisplayIsAsleep(did))
+                log("check_display() CGDisplayIsAsleep(%#x)=%s", did, asleep)
+                if self.display_is_asleep!=asleep:
+                    self.display_is_asleep = asleep
+                    if asleep:
+                        self.client.suspend()
+                    else:
+                        self.client.resume()
+        except Exception:
+            log.error("Error checking display sleep status", exc_info=True)
+            self.check_display_timer = 0
+            return False
 
     def run(self):
         #this is for running standalone
