@@ -1,18 +1,18 @@
 # This file is part of Xpra.
-# Copyright (C) 2011-2014 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2011-2016 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
-import os
-
 from xpra.gtk_common.gobject_compat import import_gtk, import_glib
 gtk = import_gtk()
 glib = import_glib()
 
+from xpra.util import envbool, csv
 from xpra.gtk_common.gtk_util import scaled_image
 from xpra.gtk_common.about import about
-from xpra.client.gtk_base.gtk_tray_menu_base import GTKTrayMenuBase, populate_encodingsmenu, CLIPBOARD_LABEL_TO_NAME, CLIPBOARD_LABELS
-from xpra.platform.darwin.osx_clipboard import OSXClipboardProtocolHelper
+from xpra.client.gtk_base.gtk_tray_menu_base import GTKTrayMenuBase, populate_encodingsmenu, \
+            CLIPBOARD_LABEL_TO_NAME, CLIPBOARD_NAME_TO_LABEL, CLIPBOARD_LABELS, CLIPBOARD_DIRECTION_LABELS, CLIPBOARD_DIRECTION_NAME_TO_LABEL, \
+            SHOW_UPLOAD, SHOW_VERSION_CHECK, RUNCOMMAND_MENU, SHOW_SERVER_COMMANDS, SHOW_SHUTDOWN
 from xpra.platform.paths import get_icon
 from xpra.platform.darwin.gui import get_OSXApplication
 
@@ -28,10 +28,11 @@ SHOW_ENCODINGS_MENU = True
 SHOW_ACTIONS_MENU = True
 SHOW_INFO_MENU = True
 SHOW_CLIPBOARD_MENU = True
+SHOW_SERVER_MENU = True
 
 SHOW_ABOUT_XPRA = True
 
-SINGLE_MENU = os.environ.get("XPRA_OSX_SINGLE_MENU", "1")=="1"
+SINGLE_MENU = envbool("XPRA_OSX_SINGLE_MENU", True)
 
 
 SEPARATOR = "SEPARATOR"
@@ -74,7 +75,11 @@ class OSXMenuHelper(GTKTrayMenuBase):
         #if we call add_about before the main loop is ready,
         #things don't work...
         if client and SHOW_ABOUT_XPRA:
-            self.client.after_handshake(self.add_about)
+            client.after_handshake(self.add_about)
+
+    def show_menu(self, button, time):
+        #does not mean anything on OSX since the menu is controlled by the OS
+        pass
 
     def build(self):
         log("OSXMenuHelper.build()")
@@ -163,10 +168,9 @@ class OSXMenuHelper(GTKTrayMenuBase):
 
     def add_full_menu(self):
         log("OSXMenuHelper.add_full_menu()")
-        if self.full:
+        if self.full or not self.client:
             return
         self.full = True
-        assert self.client
         menus = self.get_extra_menus()
         for label, submenu in reversed(menus):
             self.add_top_level_menu(label, submenu)
@@ -177,6 +181,8 @@ class OSXMenuHelper(GTKTrayMenuBase):
         if SHOW_INFO_MENU:
             info_menu = self.make_menu()
             info_menu.append(self.make_sessioninfomenuitem())
+            if SHOW_VERSION_CHECK:
+                info_menu.append(self.make_updatecheckmenuitem())
             info_menu.append(self.make_bugreportmenuitem())
             menus.append(("Info", info_menu))
         if SHOW_FEATURES_MENU:
@@ -186,6 +192,7 @@ class OSXMenuHelper(GTKTrayMenuBase):
             features_menu.add(self.make_cursorsmenuitem())
             features_menu.add(self.make_notificationsmenuitem())
             features_menu.add(self.make_swapkeysmenuitem())
+            features_menu.add(self.make_invertmousewheelmenuitem())
             features_menu.add(self.make_numlockmenuitem())
             features_menu.add(self.make_openglmenuitem())
             features_menu.add(self.make_scalingmenuitem())
@@ -193,8 +200,11 @@ class OSXMenuHelper(GTKTrayMenuBase):
             clipboard_menu = self.make_menu()
             menus.append(("Clipboard", clipboard_menu))
             for label in CLIPBOARD_LABELS:
-                remote_clipboard = CLIPBOARD_LABEL_TO_NAME[label]
-                clipboard_menu.add(self.make_clipboard_submenuitem(label, remote_clipboard))
+                clipboard_menu.add(self.make_clipboard_submenuitem(label, self._remote_clipboard_changed))
+            clipboard_menu.add(gtk.SeparatorMenuItem())
+            for label in CLIPBOARD_DIRECTION_LABELS:
+                clipboard_menu.add(self.make_clipboard_submenuitem(label, self._clipboard_direction_changed))
+            clipboard_menu.show_all()
             self.client.after_handshake(self.set_clipboard_menu, clipboard_menu)
         if SHOW_SOUND_MENU:
             sound_menu = self.make_menu()
@@ -205,64 +215,56 @@ class OSXMenuHelper(GTKTrayMenuBase):
             menus.append(("Sound", sound_menu))
         if SHOW_ENCODINGS_MENU:
             encodings_menu = self.make_menu()
-            def set_encodings_menu(*args):
+            def set_encodings_menu(*_args):
                 from xpra.codecs.loader import PREFERED_ENCODING_ORDER
+                server_encodings = list(self.client.server_encodings)
                 encodings = [x for x in PREFERED_ENCODING_ORDER if x in self.client.get_encodings()]
-                populate_encodingsmenu(encodings_menu, self.get_current_encoding, self.set_current_encoding, encodings, self.client.server_encodings)
+                if self.client.server_auto_video_encoding:
+                    encodings.insert(0, "auto")
+                    server_encodings.insert(0, "auto")
+                populate_encodingsmenu(encodings_menu, self.get_current_encoding, self.set_current_encoding, encodings, server_encodings)
             self.client.after_handshake(set_encodings_menu)
             menus.append(("Encoding", encodings_menu))
         if SHOW_ACTIONS_MENU:
             actions_menu = self.make_menu()
             actions_menu.add(self.make_refreshmenuitem())
             actions_menu.add(self.make_raisewindowsmenuitem())
+            menus.append(("Actions", actions_menu))
+        if RUNCOMMAND_MENU or SHOW_SERVER_COMMANDS or SHOW_UPLOAD or SHOW_SHUTDOWN:
+            server_menu = self.make_menu()
+            if SHOW_SHUTDOWN:
+                server_menu.append(self.make_shutdownmenuitem())
             #set_sensitive(bool) does not work on OSX,
             #so we only add the menu item if it does something
-            def add_ah(*args):
-                if self.client.start_new_commands:
-                    actions_menu.add(self.make_startnewcommandmenuitem())
-                if self.client.server_file_transfer:
-                    actions_menu.add(self.make_uploadmenuitem())
+            def add_ah(*_args):
+                if self.client.server_start_new_commands:
+                    server_menu.add(self.make_runcommandmenuitem())
+                if SHOW_SERVER_COMMANDS and self.client.server_commands_info:
+                    server_menu.append(self.make_servercommandsmenuitem())
+                if SHOW_UPLOAD and self.client.remote_file_transfer:
+                    server_menu.add(self.make_uploadmenuitem())
             self.client.after_handshake(add_ah)
-            menus.append(("Actions", actions_menu))
+            menus.append(("Server", server_menu))
         menus.append((SEPARATOR+"-EXTRAS", None))
         return menus
 
-    def get_clipboard_helper_class(self):
-        return OSXClipboardProtocolHelper
 
-    def make_clipboard_submenuitem(self, label, selection):
-        clipboard_item = self.checkitem(label)
-        clipboard_item.set_draw_as_radio(True)
-        def remote_clipboard_changed(item):
-            clipboardlog("remote_clipboard_changed(%s) label=%s - clipboard_change_pending=%s", item, label, self._clipboard_change_pending)
-            self.select_clipboard_menu_option(item, label)
-        clipboard_item.connect("toggled", remote_clipboard_changed)
-        return clipboard_item
+    def _clipboard_direction_changed(self, item, label):
+        clipboardlog("_clipboard_direction_changed(%s, %s) clipboard_change_pending=%s", item, label, self._clipboard_change_pending)
+        label = self.select_clipboard_menu_option(item, label, CLIPBOARD_DIRECTION_LABELS)
+        self.do_clipboard_direction_changed(label or "")
 
-    def select_clipboard_menu_option(self, item=None, label="Disabled"):
-        clipboardlog("select_clipboard_menu_option(%s, %s) clipboard_change_pending=%s", item, label, self._clipboard_change_pending)
-        if self._clipboard_change_pending:
+    def _remote_clipboard_changed(self, item, label):
+        clipboardlog("_remote_clipboard_changed(%s, %s) clipboard_change_pending=%s", item, label, self._clipboard_change_pending)
+        #ensure this is the only clipboard label selected:
+        label = self.select_clipboard_menu_option(item, label, CLIPBOARD_LABELS)
+        if not label:
             return
-        clipboard = self.get_menu("Clipboard")
-        if not clipboard:
-            log.error("Error: cannot locate Clipboad menu object!")
-            return
-        all_items = clipboard.get_submenu().get_children()
-        selected_items = [x for x in all_items if x==item] + [x for x in all_items if x.get_label()==label]
-        default_items = [x for x in all_items if x.get_label()=="Disabled"]
-        options = selected_items+default_items
-        if not options:
-            log.error("Error: cannot find any clipboard menu options to match!")
-            return
-        self._clipboard_change_pending = True
-        sel = options[0]
-        remote_clipboard = CLIPBOARD_LABEL_TO_NAME.get(sel.get_label())
-        clipboardlog("will select clipboard menu item with label=%s, for remote_clipboard=%s", sel.get_label(), remote_clipboard)
-        for x in all_items:
-            x.set_active(x.get_label()==sel.get_label())
-        glib.timeout_add(0, self.do_clipboard_change, remote_clipboard)
+        remote_clipboard = CLIPBOARD_LABEL_TO_NAME[label]
+        clipboardlog("will select clipboard menu item with label=%s, for remote_clipboard=%s", label, remote_clipboard)
+        glib.timeout_add(0, self._do_clipboard_change, remote_clipboard)
 
-    def do_clipboard_change(self, remote_clipboard):
+    def _do_clipboard_change(self, remote_clipboard):
         #why do we look it up again when we could just pass it in
         #to make_clipboard_submenuitem as an extra argument?
         #because gtk-osx would fall over itself, making a complete mess of the menus in the process
@@ -270,22 +272,58 @@ class OSXMenuHelper(GTKTrayMenuBase):
         self._clipboard_change_pending = False
         self.set_new_remote_clipboard(remote_clipboard)
 
-    def set_clipboard_menu(self, clipboard_menu):
+    def make_clipboard_submenuitem(self, label, cb=None):
+        clipboard_item = self.checkitem(label)
+        clipboard_item.set_draw_as_radio(True)
+        def clipboard_option_changed(item):
+            clipboardlog("clipboard_option_changed(%s) label=%s, callback=%s clipboard_change_pending=%s", item, label, cb, self._clipboard_change_pending)
+            if cb:
+                cb(item, label)
+        clipboard_item.connect("toggled", clipboard_option_changed)
+        return clipboard_item
+
+    def select_clipboard_menu_option(self, item=None, label=None, labels=[]):
+        #ensure that only the matching menu item is selected,
+        #(can be specified as a menuitem object, or using its label)
+        #all the other menu items whose labels are specified will be made inactive
+        #(we use a flag to prevent reentry)
+        clipboardlog("select_clipboard_menu_option(%s, %s, %s) clipboard_change_pending=%s", item, label, labels, self._clipboard_change_pending)
+        if self._clipboard_change_pending:
+            return None
+        clipboard = self.get_menu("Clipboard")
+        if not clipboard:
+            log.error("Error: cannot locate Clipboard menu object!")
+            return None
+        all_items = [x for x in clipboard.get_submenu().get_children() if x.get_label() in labels]
+        selected_items = [x for x in all_items if x==item] + [x for x in all_items if x.get_label()==label]
+        if not selected_items:
+            log.error("Error: cannot find any clipboard menu options to match '%s'", label)
+            log.error(" all menu items: %s", csv(x.get_label() for x in all_items))
+            log.error(" selected: %s", csv(x.get_label() for x in selected_items))
+            return None
+        self._clipboard_change_pending = True
+        sel = selected_items[0]
+        if not label:
+            label = sel.get_label()
+        for x in all_items:
+            active = x.get_label()==label
+            if x.get_active()!=active:
+                x.set_active(active)
+        self._clipboard_change_pending = False
+        return label
+
+    def set_clipboard_menu(self, _clipboard_menu):
         #find the menu item matching the current settings,
         #and select it
         try:
-            assert self._can_handle_clipboard()
-            remote_clipboard = self.client.clipboard_helper.remote_clipboard
-            label = None
+            label = CLIPBOARD_NAME_TO_LABEL.get(self.client.clipboard_helper.remote_clipboard)
+            self.select_clipboard_menu_option(None, label, CLIPBOARD_LABELS)
         except:
-            remote_clipboard = None
-            label = "Disabled"
-        selected_menu_item = None
-        for item in clipboard_menu.get_children():
-            if remote_clipboard and remote_clipboard==CLIPBOARD_LABEL_TO_NAME.get(item.get_label()):
-                selected_menu_item = item
-                break
-        self.select_clipboard_menu_option(selected_menu_item, label)
+            pass
+        direction_label = CLIPBOARD_DIRECTION_NAME_TO_LABEL.get(self.client.client_clipboard_direction, "Disabled")
+        clipboardlog("direction(%s)=%s", self.client.client_clipboard_direction, direction_label)
+        self.select_clipboard_menu_option(None, direction_label, CLIPBOARD_DIRECTION_LABELS)
+
 
     #these methods are called by the superclass
     #but we don't have a quality or speed menu, so override and ignore
@@ -309,7 +347,21 @@ class OSXMenuHelper(GTKTrayMenuBase):
                 log("set_swapkeys_menuitem(%s) no keyboard!", args)
                 self.swapkeys_menuitem.set_sensitive(False)
         self.client.after_handshake(set_swapkeys_menuitem)
-        return  self.swapkeys_menuitem
+        return self.swapkeys_menuitem
+
+    def make_invertmousewheelmenuitem(self):
+        def invert_toggled(*args):
+            v = self.mousewheel_menuitem.get_active()
+            log.info("invert_toggled(%s) invert enabled=%s", args, v)
+            if v:
+                self.client.wheel_map[4] = 5
+                self.client.wheel_map[5] = 4
+            else:
+                self.client.wheel_map[4] = 4
+                self.client.wheel_map[5] = 5
+        self.mousewheel_menuitem = self.checkitem("Invert Mouse Wheel", invert_toggled)
+        self.mousewheel_menuitem.set_active(self.client.wheel_map.get(4)!=4)
+        return self.mousewheel_menuitem
 
     def make_numlockmenuitem(self):
         def numlock_toggled(*args):
