@@ -1,12 +1,13 @@
-# coding=utf8
+# -*- coding: utf-8 -*-
 # This file is part of Xpra.
-# Copyright (C) 2013, 2014 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2013-2017 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 from xpra.log import Logger
 log = Logger("shadow", "osx")
 
+from xpra.util import envbool
 from xpra.server.gtk_server_base import GTKServerBase
 from xpra.server.shadow.root_window_model import RootWindowModel
 from xpra.server.shadow.gtk_shadow_server_base import GTKShadowServerBase
@@ -15,6 +16,9 @@ from xpra.platform.darwin.gui import get_CG_imagewrapper, take_screenshot
 
 import Quartz.CoreGraphics as CG    #@UnresolvedImport
 
+
+USE_TIMER = envbool("XPRA_OSX_SHADOW_USE_TIMER", False)
+
 ALPHA = {
          CG.kCGImageAlphaNone                  : "AlphaNone",
          CG.kCGImageAlphaPremultipliedLast     : "PremultipliedLast",
@@ -28,8 +32,8 @@ ALPHA = {
 #ensure that picture_encode can deal with pixels as NSCFData:
 def patch_picture_encode():
     from CoreFoundation import CFDataGetBytes, CFDataGetLength  #@UnresolvedImport
-    from xpra.os_util import memoryview_to_bytes, _memoryview, _buffer
-    btypes = [x for x in (str, bytes, _memoryview, _buffer, bytearray) if x is not None]
+    from xpra.os_util import memoryview_to_bytes, _buffer
+    btypes = [x for x in (str, bytes, memoryview, _buffer, bytearray) if x is not None]
     def pixels_to_bytes(v):
         if type(v) in btypes:
             return memoryview_to_bytes(v)
@@ -39,12 +43,27 @@ def patch_picture_encode():
     picture_encode.pixels_to_bytes = pixels_to_bytes
 
 
-class OSXRootWindowModel(RootWindowModel):
+class OSXRootCapture(object):
 
-    def get_image(self, x, y, width, height, logger=None):
+    def refresh(self):
+        return True
+
+    def clean(self):
+        pass
+
+    def get_image(self, x, y, width, height):
         rect = (x, y, width, height)
         return get_CG_imagewrapper(rect)
 
+    def get_info(self):
+        return {}
+
+
+class OSXRootWindowModel(RootWindowModel):
+
+    def __init__(self, root_window):
+        RootWindowModel.__init__(self, root_window, OSXRootCapture())
+
     def take_screenshot(self):
         log("grabbing screenshot")
         return take_screenshot()
@@ -79,64 +98,67 @@ class ShadowServer(GTKShadowServerBase):
 
     def make_tray_widget(self):
         from xpra.client.gtk_base.statusicon_tray import GTKStatusIconTray
-        return GTKStatusIconTray(self, self.tray, "Xpra Shadow Server", None, None, self.tray_click_callback, mouseover_cb=None, exit_cb=self.tray_exit_callback)
+        return GTKStatusIconTray(self, 0, self.tray, "Xpra Shadow Server", None, None, self.tray_click_callback, mouseover_cb=None, exit_cb=self.tray_exit_callback)
 
-    def makeRootWindowModel(self):
-        return  OSXRootWindowModel(self.root)
+    def makeRootWindowModels(self):
+        return (OSXRootWindowModel(self.root),)
 
 
-    def screen_refresh_callback(self, count, rects, info):
-        #log("screen_refresh_callback%s mapped=%s", (count, rects, info), self.mapped_at)
-        if not self.mapped_at:
-            return
+    def screen_refresh_callback(self, _count, rects, info):
+        #log("screen_refresh_callback%s mapped=%s", (count, rects, info), self.mapped)
         self.refresh_count += 1
         rlist = []
         for r in rects:
-            assert isinstance(r, CG.CGRect), "invalid rectangle in list: %s" % r
+            if not isinstance(r, CG.CGRect):
+                log.error("Error: invalid rectangle in refresh list: %s", r)
+                continue
             self.refresh_rectangle_count += 1
             rlist.append((r.origin.x, r.origin.y, r.size.width, r.size.height))
-            #return quickly, and process the list copy via idle add:
-            self.idle_add(self.do_screen_refresh, rlist)
+        #return quickly, and process the list copy via idle add:
+        self.idle_add(self.do_screen_refresh, rlist)
 
     def do_screen_refresh(self, rlist):
         #TODO: improve damage method to handle lists directly:
+        assert len(self._id_to_window)==1, "cannot handle more than one root window"
+        rwm = tuple(self._id_to_window.values())[0]
         for x, y, w, h in rlist:
-            self._damage(self.root_window_model, x, y, w, h)
+            self._damage(rwm, int(x), int(y), int(w), int(h))
 
     def start_refresh(self):
         #don't use the timer, get damage notifications:
         if self.refresh_registered:
             log.warn("Warning: screen refresh callback already registered!")
             return
-        err = CG.CGRegisterScreenRefreshCallback(self.screen_refresh_callback, None)
-        log("CGRegisterScreenRefreshCallback(%s)=%s", self.screen_refresh_callback, err)
-        if err!=0:
-            log.warn("Warning: CGRegisterScreenRefreshCallback failed with error %i", err)
-            log.warn(" using fallback timer method")
-            GTKShadowServerBase.start_refresh(self)
-        else:
-            self.refresh_registered = True
+        if not USE_TIMER:
+            err = CG.CGRegisterScreenRefreshCallback(self.screen_refresh_callback, None)
+            log("CGRegisterScreenRefreshCallback(%s)=%s", self.screen_refresh_callback, err)
+            if err==0:
+                self.refresh_registered = True
+                return
+            else:
+                log.warn("Warning: CGRegisterScreenRefreshCallback failed with error %i", err)
+                log.warn(" using fallback timer method")
+        GTKShadowServerBase.start_refresh(self)
 
     def stop_refresh(self):
-        log("stop_refresh() mapped_at=%s, timer=%s", self.mapped_at, self.timer)
+        log("stop_refresh() mapped=%s, timer=%s", self.mapped, self.refresh_timer)
         if self.refresh_registered:
-            self.mapped_at = None
             try:
                 err = CG.CGUnregisterScreenRefreshCallback(self.screen_refresh_callback, None)
                 log("CGUnregisterScreenRefreshCallback(%s)=%s", self.screen_refresh_callback, err)
                 if err:
                     log.warn(" unregistering the existing one returned %s", {0 : "OK"}.get(err, err))
-                else:
-                    self.refresh_registered = False
             except ValueError as e:
                 log.warn("Error unregistering screen refresh callback:")
                 log.warn(" %s", e)
-        else:
-            #may stop the timer fallback:
-            GTKShadowServerBase.stop_refresh(self)
+            self.refresh_registered = False
+        #may stop the timer fallback:
+        GTKShadowServerBase.stop_refresh(self)
 
 
-    def do_process_mouse_common(self, proto, wid, pointer):
+    def do_process_mouse_common(self, proto, wid, pointer, *_args):
+        assert proto in self._server_sources
+        assert wid in self._id_to_window
         CG.CGWarpMouseCursorPosition(pointer)
 
     def fake_key(self, keycode, press):
@@ -148,12 +170,12 @@ class ShadowServer(GTKShadowServerBase):
         #this causes crashes, don't do it!
         #CG.CFRelease(e)
 
-    def _process_button_action(self, proto, packet):
-        wid, button, pressed, pointer, modifiers = packet[1:6]
-        log("process_button_action(%s, %s)", proto, packet)
+    def do_process_button_action(self, proto, wid, button, pressed, pointer, modifiers, *args):
         self._update_modifiers(proto, wid, modifiers)
-        self._process_mouse_common(proto, wid, pointer)
-        pointer = self._adjust_pointer(pointer)
+        pointer = self._process_mouse_common(proto, wid, pointer)
+        self.button_action(pointer, button, pressed, -1, *args)
+
+    def button_action(self, pointer, button, pressed, _deviceid=-1, *args):
         if button<=3:
             #we should be using CGEventCreateMouseEvent
             #instead we clear previous clicks when a "higher" button is pressed... oh well
@@ -184,11 +206,12 @@ class ShadowServer(GTKShadowServerBase):
         capabilities["server_type"] = "Python/gtk2/osx-shadow"
         return capabilities
 
-    def get_info(self, proto):
+    def get_info(self, proto, *_args):
         info = GTKServerBase.get_info(self, proto)
         info.setdefault("features", {})["shadow"] = True
         info.setdefault("server", {})["type"] = "Python/gtk2/osx-shadow"
         info.setdefault("damage", {}).update({
+                                              "use-timer"       : USE_TIMER,
                                               "notifications"   : self.refresh_registered,
                                               "count"           : self.refresh_count,
                                               "rectangles"      : self.refresh_rectangle_count,
