@@ -32,19 +32,19 @@ def do_ready():
 
 
 #defaults:
-def get_native_tray_menu_helper_classes():
+def get_native_tray_menu_helper_class():
     #classes that generate menus for xpra's system tray
     #let the toolkit classes use their own
-    return []
-def get_native_tray_classes(*args):
+    return None
+def get_native_tray_classes(*_args):
     #the classes we can use for our system tray:
     #let the toolkit classes use their own
     return []
-def get_native_system_tray_classes(*args):
+def get_native_system_tray_classes(*_args):
     #the classes we can use for application system tray forwarding:
     #let the toolkit classes use their own
     return []
-def system_bell(*args):
+def system_bell(*_args):
     #let the toolkit classes use their own
     return False
 def get_native_notifier_classes():
@@ -87,8 +87,6 @@ def get_display_icc_info():
     return {}
 
 def get_icc_info():
-    from xpra.log import Logger
-    log = Logger("platform")
     ENV_ICC_DATA = os.environ.get("XPRA_ICC_DATA")
     if ENV_ICC_DATA:
         import binascii
@@ -96,6 +94,8 @@ def get_icc_info():
                 "source"    : "environment-override",
                 "data"      : binascii.unhexlify(ENV_ICC_DATA),
                 }
+    from xpra.os_util import get_util_logger
+    log = get_util_logger()
     info = {}
     try:
         from PIL import ImageCms
@@ -171,7 +171,7 @@ def get_fixed_cursor_size():
 def get_cursor_size():
     return -1
 
-def get_window_frame_size(x, y, w, h):
+def get_window_frame_size(_x, _y, _w, _h):
     return None
 
 def get_window_frame_sizes():
@@ -218,7 +218,7 @@ def get_info_base():
     def fnames(l):
         return [fname(x) for x in l]
     return {
-            "native_tray_menu_helpers"      : fnames(get_native_tray_menu_helper_classes()),
+            "native_tray_menu_helper"       : fname(get_native_tray_menu_helper_class()),
             "native_trays"                  : fnames(get_native_tray_classes()),
             "native_system_trays"           : fnames(get_native_system_tray_classes()),
             "system_bell"                   : fname(system_bell),
@@ -261,7 +261,7 @@ platform_import(globals(), "gui", False,
                 "ClientExtras",
                 "take_screenshot",
                 "get_menu_support_function",
-                "get_native_tray_menu_helper_classes",
+                "get_native_tray_menu_helper_class",
                 "get_native_tray_classes",
                 "get_native_system_tray_classes",
                 "get_native_notifier_classes",
