@@ -39,19 +39,23 @@ frozen = getattr(sys, 'frozen', False)
 REDIRECT_OUTPUT = envbool("XPRA_REDIRECT_OUTPUT", frozen is True and GetConsoleCP()==0)
 if frozen:
     #cx_freeze paths:
-    def jedir(relpathname):
-        return os.path.join(os.path.dirname(sys.executable), relpathname)
-    def addsyspath(relpathname):
-        v = jedir(relpathname)
+    edir = os.path.dirname(sys.executable)
+    def jedir(*paths):
+        return "\\".join([edir]+list(paths))
+    def addsyspath(*paths):
+        v = jedir(*paths)
         if os.path.exists(v):
             sys.path.append(v)
     addsyspath('')
     addsyspath('bin')
-    addsyspath('bin\\etc')
-    addsyspath('bin\\lib')
-    addsyspath('bin\\share')
-    addsyspath('bin\\library.zip')
-    os.environ['GI_TYPELIB_PATH'] = jedir('lib\\girepository-1.0')
+    addsyspath('bin', 'etc')
+    addsyspath('bin', 'lib')
+    addsyspath('bin', 'share')
+    addsyspath('bin', 'library.zip')
+    os.environ['GI_TYPELIB_PATH'] = jedir('lib', 'girepository-1.0')
+
+#don't know why this breaks with Python 3 yet...
+FIX_UNICODE_OUT = envbool("XPRA_FIX_UNICODE_OUT", not REDIRECT_OUTPUT and PYTHON2)
 
 
 def is_wine():
@@ -74,8 +78,9 @@ def set_prgname(name):
     prg_name = name
     try:
         SetConsoleTitleA(name)
-        import glib
-        glib.set_prgname(name)
+        if PYTHON2:
+            import glib     #@UnresolvedImport
+            glib.set_prgname(name)
     except:
         pass
 
@@ -91,7 +96,9 @@ def fix_unicode_out():
     # So be paranoid about catching errors and reporting them to original_stderr,
     # so that we can at least see them.
     def _complain(message):
-        print >>original_stderr, message if isinstance(message, str) else repr(message)
+        if not isinstance(message, str):
+            message = repr(message)
+        original_stderr.write("%s\n" % message)
 
     # Work around <http://bugs.python.org/issue6058>.
     codecs.register(lambda name: codecs.lookup('utf-8') if name == 'cp65001' else None)
@@ -169,8 +176,9 @@ def fix_unicode_out():
                         try:
                             self._stream.flush()
                         except Exception as e:
-                            _complain("%s.flush: %r from %r" % (self.name, e, self._stream))
-                            raise
+                            if not self.closed:
+                                _complain("%s.flush: %r from %r" % (self.name, e, self._stream))
+                                raise
 
                 def write(self, text):
                     try:
@@ -195,16 +203,18 @@ def fix_unicode_out():
                                     break
                                 text = text[n.value:]
                     except Exception as e:
-                        _complain("%s.write: %r" % (self.name, e))
-                        raise
+                        if not self.closed:
+                            _complain("%s.write: %r" % (self.name, e))
+                            raise
 
                 def writelines(self, lines):
                     try:
                         for line in lines:
                             self.write(line)
                     except Exception as e:
-                        _complain("%s.writelines: %r" % (self.name, e))
-                        raise
+                        if not self.closed:
+                            _complain("%s.writelines: %r" % (self.name, e))
+                            raise
 
             if real_stdout:
                 sys.stdout = UnicodeOutput(hStdout, None, STDOUT_FILENO, '<Unicode console stdout>')
@@ -272,17 +282,15 @@ def set_wait_for_input():
                 print("error accessing console: %s" % e)
 
 def do_init():
+    if FIX_UNICODE_OUT:
+        fix_unicode_out()
     if not REDIRECT_OUTPUT:
-        FIX_UNICODE_OUT = envbool("XPRA_FIX_UNICODE_OUT", True)
-        if PYTHON2 and FIX_UNICODE_OUT:
-            #don't know why this breaks with Python 3 yet...
-            fix_unicode_out()
         #figure out if we want to wait for input at the end:
         set_wait_for_input()
         return
     from xpra.platform import get_prgname
     LOG_FILENAME = (get_prgname() or "Xpra")+".log"
-    from paths import _get_data_dir
+    from xpra.platform.win32.paths import _get_data_dir
     d = _get_data_dir()
     log_file = os.path.join(d, LOG_FILENAME)
     sys.stdout = open(log_file, "a")
@@ -295,7 +303,6 @@ MB_SYSTEMMODAL      = 0x00001000
 def _show_message(message, uType):
     global prg_name
     #TODO: detect cx_freeze equivallent
-    from xpra.util import envbool
     SHOW_MESSAGEBOX = envbool("XPRA_MESSAGEBOX", True)
     if SHOW_MESSAGEBOX and REDIRECT_OUTPUT:
         #try to use an alert box since no console output will be shown:
@@ -313,13 +320,16 @@ def command_error(message):
     _show_message(message, MB_ICONEXCLAMATION | MB_SYSTEMMODAL)
 
 
-def get_main_fallback():
+def get_main_fallback(cmd):
     set_wait_for_input()
     global _wait_for_input
     if _wait_for_input:
         #something will be shown in a console,
         #so don't bother showing anything else
         return None
+    if cmd.find("Xpra_Audio.exe")>=0:
+        #don't show the launcher when trying to use this command
+        return None
     #try the launcher (better than nothing!)
     try:
         from xpra.client.gtk_base.client_launcher import main
