@@ -19,6 +19,7 @@ mouselog = Logger("win32", "mouse")
 
 from xpra.platform.win32 import constants as win32con
 from xpra.platform.win32.window_hooks import Win32Hooks
+from xpra.platform.win32.win32_events import KNOWN_EVENTS, POWER_EVENTS
 from xpra.platform.win32.common import (GetSystemMetrics, SetWindowLongW, GetWindowLongW,
                                         ClipCursor, GetCursorPos,
                                         GetDC, ReleaseDC,
@@ -31,41 +32,40 @@ from xpra.platform.win32.common import (GetSystemMetrics, SetWindowLongW, GetWin
                                         UnhookWindowsHookEx, CallNextHookEx, SetWindowsHookExA,
                                         SetConsoleCtrlHandler,
                                         GetDeviceCaps,
+                                        GetIntSystemParametersInfo,
+                                        GetUserObjectInformationA, OpenInputDesktop, CloseDesktop,
                                         user32)
 from xpra.util import AdHocStruct, csv, envint, envbool
-from xpra.os_util import monotonic_time, PYTHON2
+from xpra.os_util import PYTHON2, PYTHON3
 
 CONSOLE_EVENT_LISTENER = envbool("XPRA_CONSOLE_EVENT_LISTENER", True)
 USE_NATIVE_TRAY = envbool("XPRA_USE_NATIVE_TRAY", True)
+REINIT_VISIBLE_WINDOWS = envbool("XPRA_WIN32_REINIT_VISIBLE_WINDOWS", True)
+SCREENSAVER_LISTENER_POLL_DELAY = envint("XPRA_SCREENSAVER_LISTENER_POLL_DELAY", 10)
 
 
 from ctypes import WinDLL, CFUNCTYPE, c_int, POINTER, Structure, byref, sizeof
 from ctypes.wintypes import HWND, DWORD, WPARAM, LPARAM, MSG, WCHAR, POINT, RECT
 
+if PYTHON3:
+    from ctypes import CDLL, pythonapi, c_void_p, py_object
+    PyCapsule_GetPointer = pythonapi.PyCapsule_GetPointer
+    PyCapsule_GetPointer.restype = c_void_p
+    PyCapsule_GetPointer.argtypes = [py_object]
+    log("PyCapsute_GetPointer=%s", PyCapsule_GetPointer)
+    gdkdll = CDLL("libgdk-3-0.dll")
+    log("gdkdll=%s", gdkdll)
+
+
 shell32 = WinDLL("shell32", use_last_error=True)
 try:
     from xpra.platform.win32.propsys import set_window_group    #@UnresolvedImport
-except:
-    #try using pywin32 for "Legacy" builds:
-    try:
-        from win32com.propsys import propsys    #@UnresolvedImport
-        def set_window_group(hwnd, lhandle):
-            try:
-                ps = propsys.SHGetPropertyStoreForWindow(hwnd)
-                key = propsys.PSGetPropertyKeyFromName("System.AppUserModel.ID")
-                value = propsys.PROPVARIANTType(lhandle)
-                log("win32 hooks: calling %s(%s, %s)", ps.SetValue, key, value)
-                ps.SetValue(key, value)
-            except Exception as e:
-                log("set_window_group(%s, %s)", hwnd, lhandle, exc_info=True)
-                log.error("Error: failed to set group leader '%s':", lhandle)
-                log.error(" %s", e)
-        log.info("using legacy propsys function for window grouping")
-    except ImportError as e:
-        log("propsys missing", exc_info=True)
-        log.warn("Warning: propsys support missing, window grouping is not available:")
-        log.warn(" %s", e)
-        set_window_group = None
+except ImportError as e:
+    log("propsys missing", exc_info=True)
+    log.warn("Warning: propsys support missing:")
+    log.warn(" %s", e)
+    log.warn(" window grouping is not available")
+    set_window_group = None
 
 
 try:
@@ -112,19 +112,11 @@ DPI_AWARE = envbool("XPRA_DPI_AWARE", True)
 DPI_AWARENESS = envint("XPRA_DPI_AWARENESS", 1)
 FORWARD_WINDOWS_KEY = envbool("XPRA_FORWARD_WINDOWS_KEY", True)
 WHEEL = envbool("XPRA_WHEEL", True)
-WHEEL_DELTA = envint("XPRA_WHEEL_DELTA", 120)
+WHEEL_DELTA = envint("XPRA_WIN32_WHEEL_DELTA", 120)
 assert WHEEL_DELTA>0
 
-
-KNOWN_EVENTS = {}
-POWER_EVENTS = {}
-for x in dir(win32con):
-    if x.endswith("_EVENT"):
-        v = getattr(win32con, x)
-        KNOWN_EVENTS[v] = x
-    if x.startswith("PBT_"):
-        v = getattr(win32con, x)
-        POWER_EVENTS[v] = x
+log("win32 gui settings: CONSOLE_EVENT_LISTENER=%s, USE_NATIVE_TRAY=%s, WINDOW_HOOKS=%s, GROUP_LEADER=%s, UNDECORATED_STYLE=%s, CLIP_CURSOR=%s, MAX_SIZE_HINT=%s, GEOMETRY=%s, LANGCHANGE=%s, DPI_AWARE=%s, DPI_AWARENESS=%s, FORWARD_WINDOWS_KEY=%s, WHEEL=%s, WHEEL_DELTA=%s",
+    CONSOLE_EVENT_LISTENER, USE_NATIVE_TRAY, WINDOW_HOOKS, GROUP_LEADER, UNDECORATED_STYLE, CLIP_CURSOR, MAX_SIZE_HINT, GEOMETRY, LANGCHANGE, DPI_AWARE, DPI_AWARENESS, FORWARD_WINDOWS_KEY, WHEEL, WHEEL_DELTA)
 
 
 def do_init():
@@ -181,7 +173,7 @@ def get_native_tray_classes():
             log.warn(" %s", e)
     return c
 
-def get_native_system_tray_classes(*args):
+def get_native_system_tray_classes(*_args):
     #Win32Tray cannot set the icon from data
     #so it cannot be used for application trays
     return get_native_tray_classes()
@@ -229,10 +221,30 @@ def get_window_handle(window):
         gdk_window = window.get_window()
     except:
         pass
-    try:
+    if not gdk_window:
+        return 0
+    if PYTHON2:
         return gdk_window.handle
-    except:
-        return None
+    gpointer =  PyCapsule_GetPointer(gdk_window.__gpointer__, None)
+    log("gpointer=%#x", gpointer)
+    hwnd = gdkdll.gdk_win32_window_get_handle(gpointer)
+    log("hwnd=%#x", hwnd)
+    return hwnd
+
+def get_desktop_name():
+    try:
+        desktop = OpenInputDesktop(0, True, win32con.MAXIMUM_ALLOWED)
+        if desktop:
+            buf = ctypes.create_string_buffer(128)
+            r = GetUserObjectInformationA(desktop, win32con.UOI_NAME, buf, len(buf), None)
+            if r!=0:
+                desktop_name = buf.value.decode("latin1")
+                return desktop_name
+            CloseDesktop(desktop)
+    except Exception as e:
+        log.warn("Warning: failed to get desktop name")
+        log.warn(" %s", e)
+    return None
 
 
 def get_session_type():
@@ -270,8 +282,10 @@ def win32_propsys_set_group_leader(self, leader):
         return
     try:
         log("win32_propsys_set_group_leader(%s)", leader)
-        lhandle = leader.handle
+        lhandle = get_window_handle(leader)
+        assert lhandle
     except:
+        log("win32_propsys_set_group_leader(%s)", leader, exc_info=True)
         log.warn("Warning: no window handle for %s", leader)
         log.warn(" cannot set window grouping attribute")
         return
@@ -351,7 +365,7 @@ def pointer_ungrab(window, *args):
         ClipCursor(None)
     window._client.pointer_grabbed = False
 
-def fixup_window_style(self, *args):
+def fixup_window_style(self, *_args):
     """ a fixup function we want to call from other places """
     hwnd = get_window_handle(self)
     if not hwnd:
@@ -386,6 +400,22 @@ def fixup_window_style(self, *args):
             SetWindowLongW(hwnd, win32con.GWL_STYLE, style)
         else:
             log("fixup_window_style() unchanged style %s (%#x) on window %#x", style_str(style), style, hwnd)
+        ws_visible = bool(style & win32con.WS_VISIBLE)
+        client = self._client
+        cur_ws_visible = getattr(self, "_ws_visible", True)
+        iconified = getattr(self, "_iconified", False)
+        been_mapped = getattr(self, "_been_mapped", False)
+        log("fixup_window_style() ws_visible=%s (was %s), iconified=%s, been_mapped=%s", ws_visible, cur_ws_visible, iconified, been_mapped)
+        if client and been_mapped and not iconified and ws_visible!=cur_ws_visible:
+            log("window changed visibility to: %s", ws_visible)
+            setattr(self, "_ws_visible", ws_visible)
+            if ws_visible:
+                #with opengl, we need to re-create the window (PITA):
+                if REINIT_VISIBLE_WINDOWS:
+                    client.reinit_window(self._id, self)
+                self.send_control_refresh(False)
+            else:
+                self.send_control_refresh(True)
     except:
         log.warn("failed to fixup window style", exc_info=True)
 
@@ -407,6 +437,8 @@ def apply_maxsize_hints(window, hints):
     """
     workw, workh = 0, 0
     handle = get_window_handle(window)
+    if not handle:
+        return
     log("apply_maxsize_hints(%s, %s) handle=%s", window, hints, handle)
     if not window.get_decorated():
         workarea = get_monitor_workarea_for_window(handle)
@@ -414,7 +446,7 @@ def apply_maxsize_hints(window, hints):
         if workarea:
             workw, workh = workarea[2:4]
     minw = hints.get("min_width", 0)
-    minh = hints.get("min_width", 0)
+    minh = hints.get("min_height", 0)
     maxw = hints.get("max_width", 0)
     maxh = hints.get("max_height", 0)
     if workw>0 and workh>0:
@@ -473,25 +505,7 @@ def add_window_hooks(window):
         gdk_window.set_group = noop
     except:
         gdk_window = None
-    #window handle:
-    try:
-        handle = gdk_window.handle
-    except:
-        handle = None
-    if not handle:
-        from xpra.gtk_common.gobject_compat import is_gtk3
-        if is_gtk3():
-            #access the missing gdk_win32_window_get_handle function using ctypes:
-            try:
-                ctypes.pythonapi.PyCapsule_GetPointer.restype = ctypes.c_void_p
-                ctypes.pythonapi.PyCapsule_GetPointer.argtypes = [ctypes.py_object]
-                gdkwin_gpointer = ctypes.pythonapi.PyCapsule_GetPointer(gdk_window.__gpointer__, None)
-                gdkdll = ctypes.CDLL("libgdk-3-0.dll")
-                handle = gdkdll.gdk_win32_window_get_handle(gdkwin_gpointer)
-            except Exception as e:
-                log("failed to get window handle", exc_info=True)
-                log.error("Error: failed to access the window handle for %s", gdk_window)
-                log.error(" %s", e)
+    handle = get_window_handle(gdk_window)
     if not handle:
         log.warn("Warning: cannot add window hooks without a window handle!")
         return
@@ -540,15 +554,12 @@ def add_window_hooks(window):
                 apply_maxsize_hints(window, window.geometry_hints)
 
         if LANGCHANGE:
-            def inputlangchange(hwnd, event, wParam, lParam):
+            def inputlangchange(_hwnd, _event, wParam, lParam):
                 log("WM_INPUTLANGCHANGE: character set: %i, input locale identifier: %i", wParam, lParam)
                 window.keyboard_layout_changed("WM_INPUTLANGCHANGE", wParam, lParam)
             win32hooks.add_window_event_handler(win32con.WM_INPUTLANGCHANGE, inputlangchange)
 
         if WHEEL:
-            #keep track of the pointer offsets:
-            #(difference between the GTK event values and raw win32 values)
-            window._get_pointer = types.MethodType(cache_pointer_offset, window)
             VERTICAL = "vertical"
             HORIZONTAL = "horizontal"
             def handle_wheel(orientation, wParam, lParam):
@@ -559,62 +570,27 @@ def add_window_hooks(window):
                 keys = wParam & 0xFFFF
                 y = lParam>>16
                 x = lParam & 0xFFFF
-                last_event_time = getattr(window, "_wheel_event_time", 0)
-                #only accumulate if the last event was less than 5 seconds ago:
-                if monotonic_time()-last_event_time<5:
-                    cval = getattr(window, "_win32_%swheel" % orientation, 0)
-                else:
-                    cval = 0
-                nval = cval + distance
-                units = abs(nval) // WHEEL_DELTA
+                units = float(distance) / WHEEL_DELTA
                 client = getattr(window, "_client")
                 wid = getattr(window, "_id", 0)
-                gtk_offset = getattr(window, "win32_pointer_offset", None)
-                if gtk_offset:
-                    dx, dy = gtk_offset
-                    x += dx
-                    y += dy
-                mouselog("mousewheel: orientation=%s distance=%.1f, units=%i, new value=%.1f, keys=%#x, x=%i, y=%i, gtk_offset=%s, client=%s, wid=%i", orientation, distance, units, nval, keys, x, y, gtk_offset, client, wid)
-                if units>0 and client and wid>0:
+                mouselog("win32 mousewheel: orientation=%s, distance=%i, wheel-delta=%s, units=%.3f, new value=%.1f, keys=%#x, x=%i, y=%i, client=%s, wid=%i", orientation, distance, WHEEL_DELTA, units, distance, keys, x, y, client, wid)
+                if client and wid>0:
                     if orientation==VERTICAL:
-                        button = 4 + int(nval<0)        #4 for UP, 5 for DOWN
+                        deltax = 0
+                        deltay = units
                     else:
-                        button = 7 - int(nval<0)        #6 for LEFT, 7 for RIGHT
-                    button = client.wheel_map.get(button)
-                    buttons = []
-                    modifiers = client.get_current_modifiers()
-                    pointer = window._pointer(x, y)
-                    def send_button(pressed):
-                        if button:
-                            client.send_button(wid, button, pressed, pointer, modifiers, buttons)
-                    count = 0
-                    v = nval
-                    while abs(v)>=WHEEL_DELTA:
-                        send_button(True)
-                        send_button(False)
-                        if v>0:
-                            v -= WHEEL_DELTA
-                        else:
-                            v += WHEEL_DELTA
-                        count += 1
-                    mouselog("mousewheel: sent %i wheel events to the server for button=%s, distance=%s, remainder=%s", count, button, nval, v)
-                    nval = v
-                setattr(window, "_win32_%swheel" % orientation, nval)
-                setattr(window, "_wheel_event_time", monotonic_time())
-            def mousewheel(hwnd, event, wParam, lParam):
+                        deltax = units
+                        deltay = 0
+                    client.wheel_event(wid, deltax, deltay)
+            def mousewheel(_hwnd, _event, wParam, lParam):
                 handle_wheel(VERTICAL, wParam, lParam)
                 return 0
-            def mousehwheel(hwnd, event, wParam, lParam):
+            def mousehwheel(_hwnd, _event, wParam, lParam):
                 handle_wheel(HORIZONTAL, wParam, lParam)
                 return 0
             WM_MOUSEHWHEEL = 0x020E
             win32hooks.add_window_event_handler(win32con.WM_MOUSEWHEEL, mousewheel)
             win32hooks.add_window_event_handler(WM_MOUSEHWHEEL, mousehwheel)
-            def reset_wheel_counters(*args):
-                mouselog("window lost focus, resetting current wheel deltas")
-                for orientation in (VERTICAL, HORIZONTAL):
-                    setattr(window, "_win32_%swheel" % orientation, 0)
-            window.connect("focus-out-event", reset_wheel_counters)
 
 
 def remove_window_hooks(window):
@@ -666,10 +642,9 @@ FE_FONTSMOOTHING_STR = {
 
 
 def _add_SPI(info, constant, name, convert, default=None):
-    SystemParametersInfo = user32.SystemParametersInfoA
-    i = ctypes.c_uint32()
-    if SystemParametersInfo(constant, 0, byref(i), 0):
-        info[name] = convert(i.value)
+    v = GetIntSystemParametersInfo(constant)
+    if v is not None:
+        info[name] = convert(v)
     elif default is not None:
         info[name] = default
 
@@ -809,11 +784,13 @@ def _get_device_caps(constant):
 def get_vrefresh():
     try:
         v = _get_device_caps(win32con.VREFRESH)
-        screenlog("get_vrefresh()=%s", v)
-        return v
     except Exception as e:
-        log.warn("failed to get VREFRESH: %s", e)
-        return -1
+        log("get_vrefresh()", exc_info=True)
+        log.warn("Warning: failed to query the display vertical refresh rate:")
+        log.warn(" %s", e)
+        v = -1
+    screenlog("get_vrefresh()=%s", v)
+    return v
 
 def get_double_click_time():
     try:
@@ -931,10 +908,25 @@ WTS_SESSION_EVENTS = {
                       }
 
 class ClientExtras(object):
-    def __init__(self, client, opts):
+    def __init__(self, client, _opts):
         self.client = client
         self._kh_warning = False
         self._console_handler_added = False
+        self._screensaver_state = False
+        self._screensaver_timer = 0
+        self._exit = False
+        if SCREENSAVER_LISTENER_POLL_DELAY>0:
+            def log_screensaver():
+                v = bool(GetIntSystemParametersInfo(win32con.SPI_GETSCREENSAVERRUNNING))
+                log("SPI_GETSCREENSAVERRUNNING=%s", v)
+                if self._screensaver_state!=v:
+                    self._screensaver_state = v
+                    if v:
+                        self.client.suspend()
+                    else:
+                        self.client.resume()
+                return True
+            self._screensaver_timer = client.timeout_add(SCREENSAVER_LISTENER_POLL_DELAY*1000, log_screensaver)
         if CONSOLE_EVENT_LISTENER:
             self._console_handler_added = self.setup_console_event_listener(True)
         from xpra.platform.win32.win32_events import get_win32_event_listener
@@ -949,7 +941,8 @@ class ClientExtras(object):
                 el.add_event_callback(win32con.WM_INPUTLANGCHANGE,  self.inputlangchange)
                 el.add_event_callback(win32con.WM_WININICHANGE,     self.inichange)
         except Exception as e:
-            log.error("cannot register focus and power callbacks: %s", e)
+            log.error("Error: cannot register focus and power callbacks:")
+            log.error(" %s", e)
         self.keyboard_hook_id = None
         if FORWARD_WINDOWS_KEY:
             from xpra.make_thread import make_thread
@@ -960,6 +953,7 @@ class ClientExtras(object):
 
     def cleanup(self):
         log("ClientExtras.cleanup()")
+        self._exit = True
         cha = self._console_handler_added
         if cha:
             self._console_handler_added = False
@@ -972,6 +966,10 @@ class ClientExtras(object):
         if khid:
             self.keyboard_hook_id = None
             UnhookWindowsHookEx(khid)
+        sst = self._screensaver_timer
+        if sst:
+            self._screensaver_timer = 0
+            self.client.source_remove(sst)
         log("ClientExtras.cleanup() ended")
         #self.client = None
 
@@ -1065,7 +1063,7 @@ class ClientExtras(object):
         keylog("init_keyboard_listener() hook_id=%#x", keyboard_hook_id)
         msg = MSG()
         lpmsg = byref(msg)
-        while True:
+        while not self._exit:
             ret = GetMessageA(lpmsg, None, 0, 0)
             keylog("init_keyboard_listener: GetMessage()=%s", ret)
             if ret==-1:
@@ -1190,7 +1188,8 @@ def main():
             log.enable_debug()
             win32_event_logger.enable_debug()
 
-        import gobject
+        from xpra.gtk_common.gobject_compat import import_gobject
+        gobject = import_gobject()
         gobject.threads_init()      #@UndefinedVariable
 
         log.info("Event loop is running")
@@ -1207,7 +1206,7 @@ def main():
         fake_client.suspend = suspend
         fake_client.resume = resume
         fake_client.keyboard_helper = None
-        def signal_quit(*args):
+        def signal_quit(*_args):
             loop.quit()
         fake_client.signal_disconnect_and_quit = signal_quit
         ClientExtras(fake_client, None)
