@@ -1,30 +1,34 @@
-# coding=utf8
+# -*- coding: utf-8 -*-
 # This file is part of Xpra.
-# Copyright (C) 2012-2017 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2012-2018 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 import os
 import ctypes
 
-from ctypes.wintypes import RECT
+from ctypes.wintypes import RECT, POINT, BYTE, MAX_PATH
 
 from xpra.log import Logger
-from xpra.util import AdHocStruct, envbool, prettify_plug_name
+from xpra.os_util import strtobytes
+from xpra.util import envbool, prettify_plug_name, csv
 log = Logger("shadow", "win32")
 traylog = Logger("tray")
 shapelog = Logger("shape")
+cursorlog = Logger("cursor")
 netlog = Logger("network")
+keylog = Logger("keyboard")
 
-from xpra.util import XPRA_APP_ID
+from collections import namedtuple
+from xpra.util import XPRA_APP_ID, XPRA_IDLE_NOTIFICATION_ID
 from xpra.scripts.config import InitException
-from xpra.codecs.codec_constants import CodecStateException, TransientCodecException
 from xpra.server.gtk_server_base import GTKServerBase
 from xpra.server.shadow.gtk_shadow_server_base import GTKShadowServerBase
 from xpra.server.shadow.root_window_model import RootWindowModel
 from xpra.platform.win32 import constants as win32con
+from xpra.platform.win32.gui import get_desktop_name, get_fixed_cursor_size
 from xpra.platform.win32.keyboard_config import KeyboardConfig, fake_key
-from xpra.platform.win32.win32_events import get_win32_event_listener
+from xpra.platform.win32.win32_events import get_win32_event_listener, POWER_EVENTS
 from xpra.platform.win32.gdi_screen_capture import GDICapture
 
 #user32:
@@ -34,6 +38,13 @@ from xpra.platform.win32.common import (EnumWindows, EnumWindowsProc, FindWindow
                                         GetWindowThreadProcessId,
                                         GetSystemMetrics,
                                         SetPhysicalCursorPos,
+                                        GetPhysicalCursorPos,
+                                        GetCursorInfo, CURSORINFO,
+                                        GetDC, CreateCompatibleDC, CreateCompatibleBitmap, SelectObject, DeleteObject, ReleaseDC, DeleteDC, DrawIconEx, GetBitmapBits,
+                                        GetIconInfo, ICONINFO, Bitmap,
+                                        GetIconInfoExW, ICONINFOEXW,
+                                        GetObjectA,
+                                        GetKeyboardState, SetKeyboardState,
                                         mouse_event)
 
 NOEVENT = object()
@@ -57,12 +68,104 @@ SHADOW_GDI = envbool("XPRA_SHADOW_GDI", True)
 NVFBC_CUDA = envbool("XPRA_NVFBC_CUDA", True)
 
 
-def init_capture(pixel_depth=32):
+def get_root_window_size():
+    w = GetSystemMetrics(win32con.SM_CXVIRTUALSCREEN)
+    h = GetSystemMetrics(win32con.SM_CYVIRTUALSCREEN)
+    return w, h
+
+def get_cursor_data(hCursor):
+    #w, h = get_fixed_cursor_size()
+    x, y = 0, 0
+    dc = None
+    memdc = None
+    bitmap = None
+    old_handle = None
+    pixels = None
+    try:
+        ii = ICONINFO()
+        ii.cbSize = ctypes.sizeof(ICONINFO)
+        if not GetIconInfo(hCursor, ctypes.byref(ii)):
+            raise WindowsError()
+        x = ii.xHotspot
+        y = ii.yHotspot
+        cursorlog("get_cursor_data(%#x) hotspot at %ix%i, hbmColor=%#x, hbmMask=%#x", hCursor, x, y, ii.hbmColor, ii.hbmMask)
+        iie = ICONINFOEXW()
+        iie.cbSize = ctypes.sizeof(ICONINFOEXW)
+        if not GetIconInfoExW(hCursor, ctypes.byref(iie)):
+            raise WindowsError()
+        name = iie.szResName[:MAX_PATH]
+        cursorlog("wResID=%i, sxModName=%s, szResName=%s", iie.wResID, iie.sxModName[:MAX_PATH], name)
+        bm = Bitmap()
+        if not GetObjectA(ii.hbmColor, ctypes.sizeof(Bitmap), ctypes.byref(bm)):
+            raise WindowsError()
+        cursorlog("cursor bitmap: type=%i, width=%i, height=%i, width bytes=%i, planes=%i, bits pixel=%i, bits=%#x",
+                  bm.bmType, bm.bmWidth, bm.bmHeight, bm.bmWidthBytes, bm.bmPlanes, bm.bmBitsPixel, bm.bmBits or 0)
+        w = bm.bmWidth
+        h = bm.bmHeight
+        dc = GetDC(None)
+        assert dc, "failed to get a drawing context"
+        memdc = CreateCompatibleDC(dc)
+        assert memdc, "failed to get a compatible drawing context from %s" % dc
+        bitmap = CreateCompatibleBitmap(dc, w, h)
+        assert bitmap, "failed to get a compatible bitmap from %s" % dc
+        old_handle = SelectObject(memdc, bitmap)
+
+        #check if icon is animated:
+        UINT_MAX = 2**32-1
+        if not DrawIconEx(memdc, 0, 0, hCursor, w, h, UINT_MAX, 0, 0):
+            cursorlog("cursor is animated!")
+
+        #if not DrawIcon(memdc, 0, 0, hCursor):
+        if not DrawIconEx(memdc, 0, 0, hCursor, w, h, 0, 0, win32con.DI_NORMAL):
+            raise WindowsError()
+
+        buf_size = bm.bmWidthBytes*h
+        buf = ctypes.create_string_buffer(b"", buf_size)
+        r = GetBitmapBits(bitmap, buf_size, ctypes.byref(buf))
+        cursorlog("get_cursor_data(%#x) GetBitmapBits(%#x, %#x, %#x)=%i", hCursor, bitmap, buf_size, ctypes.addressof(buf), r)
+        if r==0:
+            cursorlog.error("Error: failed to copy screen bitmap data")
+            return None
+        elif r!=buf_size:
+            cursorlog.warn("Warning: invalid cursor buffer size, got %i bytes but expected %i", r, buf_size)
+            return None
+        else:
+            pixels = bytearray(strtobytes(buf.raw))
+            has_alpha = False
+            has_pixels = False
+            for i in range(len(pixels)//4):
+                has_pixels = has_pixels or pixels[i*4]!=0 or pixels[i*4+1]!=0 or pixels[i*4+2]!=0
+                has_alpha = has_alpha or pixels[i*4+3]!=0
+                if has_pixels and has_alpha:
+                    break
+            if has_pixels and not has_alpha:
+                #generate missing alpha - don't ask me why
+                for i in range(len(pixels)//4):
+                    if pixels[i*4]!=0 or pixels[i*4+1]!=0 or pixels[i*4+2]!=0:
+                        pixels[i*4+3] = 0xff
+        return [0, 0, w, h, x, y, hCursor, bytes(pixels), strtobytes(name)]
+    except Exception as e:
+        cursorlog("get_cursor_data(%#x)", hCursor, exc_info=True)
+        cursorlog.error("Error: failed to grab cursor:")
+        cursorlog.error(" %s", e)
+        return None
+    finally:
+        if old_handle:
+            SelectObject(memdc, old_handle)
+        if bitmap:
+            DeleteObject(bitmap)
+        if memdc:
+            DeleteDC(memdc)
+        if dc:
+            ReleaseDC(None, dc)
+
+
+def init_capture(w, h, pixel_depth=32):
     capture = None
     if SHADOW_NVFBC:
         try:
             from xpra.codecs.nvfbc.fbc_capture_win import init_nvfbc_library
-        except ImportError as e:
+        except ImportError:
             log("NvFBC capture is not available", exc_info=True)
         else:
             try:
@@ -79,12 +182,14 @@ def init_capture(pixel_depth=32):
                     else:
                         from xpra.codecs.nvfbc.fbc_capture_win import NvFBC_SysCapture
                         capture = NvFBC_SysCapture()
-                    capture.init_context(-1, -1, pixel_format)
+                    capture.init_context(w, h, pixel_format)
             except Exception as e:
+                capture = None
                 log("NvFBC_Capture", exc_info=True)
                 log.warn("Warning: NvFBC screen capture initialization failed:")
                 log.warn(" %s", e)
                 log.warn(" using the slower GDI capture code")
+                del e
     if not capture:
         if SHADOW_GDI:
             capture = GDICapture()
@@ -96,36 +201,15 @@ def init_capture(pixel_depth=32):
 
 class Win32RootWindowModel(RootWindowModel):
 
-    def __init__(self, root, pixel_depth=32):
-        RootWindowModel.__init__(self, root)
-        self.pixel_depth = pixel_depth
-        self.capture = init_capture(pixel_depth)
-        log("Win32RootWindowModel(%s, %i) capture=%s", root, pixel_depth, self.capture)
+    def __init__(self, root, capture):
+        RootWindowModel.__init__(self, root, capture)
+        log("Win32RootWindowModel(%s, %s) SEAMLESS=%s", root, capture, SEAMLESS)
         if SEAMLESS:
             self.property_names.append("shape")
             self.dynamic_property_names.append("shape")
             self.rectangles = self.get_shape_rectangles(logit=True)
             self.shape_notify = []
 
-    def cleanup(self):
-        RootWindowModel.cleanup(self)
-        self.cleanup_capture()
-
-    def cleanup_capture(self):
-        c = self.capture
-        if c:
-            self.capture = None
-            c.clean()
-
-    def get_info(self):
-        c = self.capture
-        info = {}
-        if c:
-            info["capture"] = c.get_info()
-        info["pixel-depth"] = self.pixel_depth
-        return info
-
-
     def refresh_shape(self):
         rectangles = self.get_shape_rectangles()
         if rectangles==self.rectangles:
@@ -133,8 +217,8 @@ class Win32RootWindowModel(RootWindowModel):
         self.rectangles = rectangles
         shapelog("refresh_shape() sending notify for updated rectangles: %s", rectangles)
         #notify listeners:
-        pspec = AdHocStruct()
-        pspec.name = "shape"
+        PSpec = namedtuple("PSpec", "name")
+        pspec = PSpec(name="shape")
         for cb, args in self.shape_notify:
             shapelog("refresh_shape() notifying: %s", cb)
             try:
@@ -237,31 +321,11 @@ class Win32RootWindowModel(RootWindowModel):
             return shape
         return RootWindowModel.get_property(self, prop)
 
+    def get_dimensions(self):
+        return self.get_root_window_size()
 
     def get_root_window_size(self):
-        w = GetSystemMetrics(win32con.SM_CXVIRTUALSCREEN)
-        h = GetSystemMetrics(win32con.SM_CYVIRTUALSCREEN)
-        return w, h
-
-    def get_image(self, x, y, width, height, logger=None):
-        if not self.capture:
-            self.capture = init_capture(self.pixel_depth)
-        try:
-            return self.capture.get_image(x, y, width, height)
-        except CodecStateException as e:
-            log("%s.get_image%s", self.capture, (x, y, width, height), exc_info=True)
-            #maybe we should exit here?
-            log.warn("Warning: %s", e)
-            self.cleanup_capture()
-            return None
-        except TransientCodecException as e:
-            log("%s.get_image%s", self.capture, (x, y, width, height), exc_info=True)
-            log.warn("Warning: %s", e)
-            self.cleanup_capture()
-            return None
-
-    def take_screenshot(self):
-        return self.capture.take_screenshot()
+        return get_root_window_size()
 
 
 class ShadowServer(GTKShadowServerBase):
@@ -269,6 +333,8 @@ class ShadowServer(GTKShadowServerBase):
     def __init__(self):
         GTKShadowServerBase.__init__(self)
         self.keycodes = {}
+        self.cursor_handle = None
+        self.cursor_data = None
         if GetSystemMetrics(win32con.SM_SAMEDISPLAYFORMAT)==0:
             raise InitException("all the monitors must use the same display format")
         el = get_win32_event_listener()
@@ -277,7 +343,7 @@ class ShadowServer(GTKShadowServerBase):
         #to prevent servers spinning wildly on non-blocking sockets:
         set_continue_wait(5)
         #TODO: deal with those messages?
-        #el.add_event_callback(win32con.WM_POWERBROADCAST,   self.power_broadcast_event)
+        el.add_event_callback(win32con.WM_POWERBROADCAST,   self.power_broadcast_event)
         #el.add_event_callback(WM_WTSSESSION_CHANGE,         self.session_change_event)
         #these are bound to callbacks in the client,
         #but on the server we just ignore them:
@@ -287,6 +353,10 @@ class ShadowServer(GTKShadowServerBase):
                                  win32con.WM_INPUTLANGCHANGE    : "WM_INPUTLANGCHANGE",
                                  win32con.WM_WININICHANGE       : "WM_WININICHANGE",
                                  })
+        #non-blocking server sockets (TCP and named pipes):
+        from xpra.net.bytestreams import CONTINUE_ERRNO
+        import errno
+        CONTINUE_ERRNO[errno.WSAEWOULDBLOCK] = "WSAEWOULDBLOCK"     #@UndefinedVariable
 
     def init(self, opts):
         self.pixel_depth = int(opts.pixel_depth) or 32
@@ -295,8 +365,25 @@ class ShadowServer(GTKShadowServerBase):
         GTKShadowServerBase.init(self, opts)
 
 
+    def power_broadcast_event(self, wParam, lParam):
+        log("WM_POWERBROADCAST: %s/%s", POWER_EVENTS.get(wParam, wParam), lParam)
+        if wParam==win32con.PBT_APMSUSPEND:
+            log.info("WM_POWERBROADCAST: PBT_APMSUSPEND")
+            for source in self._server_sources.values():
+                source.may_notify(XPRA_IDLE_NOTIFICATION_ID, "Server Suspending",
+                                  "This Xpra server is going to suspend,\nthe connection is likely to be interrupted soon.", expire_timeout=10*1000, icon_name="shutdown")
+        elif wParam==win32con.PBT_APMRESUMEAUTOMATIC:
+            log.info("WM_POWERBROADCAST: PBT_APMRESUMEAUTOMATIC")
+
+
+    def guess_session_name(self, _procs):
+        desktop_name = get_desktop_name()
+        if desktop_name:
+            self.session_name = desktop_name
+
+
     def print_screen_info(self):
-        w, h = self.root.get_size()
+        w, h = self.get_root_window_size()
         try:
             display = prettify_plug_name(self.root.get_screen().get_display().get_name())
         except:
@@ -309,18 +396,51 @@ class ShadowServer(GTKShadowServerBase):
         return Win32Tray(self, XPRA_APP_ID, self.tray_menu, "Xpra Shadow Server", "server-notconnected", None, self.tray_click_callback, None, self.tray_exit_callback)
 
 
-    def makeRootWindowModel(self):
-        return Win32RootWindowModel(self.root, self.pixel_depth)
+    def makeRootWindowModels(self):
+        w, h = get_root_window_size()
+        self.capture = init_capture(w, h, self.pixel_depth)
+        return (Win32RootWindowModel(self.root, self.capture),)
 
 
     def refresh(self):
         v = GTKShadowServerBase.refresh(self)
         if v and SEAMLESS:
-            self.root_window_model.refresh_shape()
+            for rwm in self._id_to_window.values():
+                rwm.refresh_shape()
         log("refresh()=%s", v)
         return v
 
-    def do_process_mouse_common(self, proto, wid, pointer, *args):
+    def do_get_cursor_data(self):
+        ci = CURSORINFO()
+        ci.cbSize = ctypes.sizeof(CURSORINFO)
+        GetCursorInfo(ctypes.byref(ci))
+        #cursorlog("GetCursorInfo handle=%#x, last handle=%#x", ci.hCursor or 0, self.cursor_handle or 0)
+        if not (ci.flags & win32con.CURSOR_SHOWING):
+            #cursorlog("do_get_cursor_data() cursor not shown")
+            return None
+        handle = int(ci.hCursor)
+        if handle==self.cursor_handle and self.last_cursor_data:
+            #cursorlog("do_get_cursor_data() cursor handle unchanged")
+            return self.last_cursor_data
+        self.cursor_handle = handle
+        cd = get_cursor_data(handle)
+        if not cd:
+            cursorlog("do_get_cursor_data() no cursor data")
+            return self.last_cursor_data
+        cd[0] = ci.ptScreenPos.x
+        cd[1] = ci.ptScreenPos.y
+        w, h = get_fixed_cursor_size()
+        return (
+            cd,
+            ((w,h), [(w,h), ]),
+            )
+
+    def get_pointer_position(self):
+        pos = POINT()
+        GetPhysicalCursorPos(ctypes.byref(pos))
+        return pos.x, pos.y
+
+    def do_process_mouse_common(self, proto, wid, pointer, *_args):
         #adjust pointer position for offset in client:
         try:
             x, y = pointer
@@ -330,6 +450,21 @@ class ShadowServer(GTKShadowServerBase):
             log.error("Error: failed to move the cursor:")
             log.error(" %s", e)
 
+    def clear_keys_pressed(self):
+        keystate = (BYTE*256)()
+        if GetKeyboardState(keystate):
+            vknames = {}
+            for vkconst in (x for x in dir(win32con) if x.startswith("VK_")):
+                vknames[getattr(win32con, vkconst)] = vkconst[3:]
+            pressed = []
+            for i in range(256):
+                if keystate[i]:
+                    pressed.append(vknames.get(i, i))
+            keylog("keys still pressed: %s", csv(pressed))
+            #clear all: (this does not affect lock modifiers: caps , num, scroll)
+            keystate = (BYTE*256)()
+            SetKeyboardState(keystate)
+
     def get_keyboard_config(self, props):
         return KeyboardConfig()
 
@@ -338,8 +473,11 @@ class ShadowServer(GTKShadowServerBase):
 
     def do_process_button_action(self, proto, wid, button, pressed, pointer, modifiers, *args):
         self._update_modifiers(proto, wid, modifiers)
-        x, y = self._process_mouse_common(proto, wid, pointer)
+        pointer = self._process_mouse_common(proto, wid, pointer)
         self._server_sources.get(proto).user_event()
+        self.button_action(pointer, button, pressed, -1, *args)
+
+    def button_action(self, pointer, button, pressed, deviceid=-1, *args):
         event = BUTTON_EVENTS.get((button, pressed))
         if event is None:
             log.warn("no matching event found for button=%s, pressed=%s", button, pressed)
@@ -347,6 +485,7 @@ class ShadowServer(GTKShadowServerBase):
         elif event is NOEVENT:
             return
         dwFlags, dwData = event
+        x, y = pointer
         mouse_event(dwFlags, x, y, dwData, 0)
 
     def make_hello(self, source):
@@ -355,7 +494,7 @@ class ShadowServer(GTKShadowServerBase):
         capabilities["server_type"] = "Python/gtk2/win32-shadow"
         return capabilities
 
-    def get_info(self, proto):
+    def get_info(self, proto, *_args):
         info = GTKServerBase.get_info(self, proto)
         info.update(GTKShadowServerBase.get_info(self, proto))
         info.setdefault("features", {})["shadow"] = True
