@@ -24,9 +24,20 @@ try:
 except Exception as e:
     log.error("Error: cannot load WtsApi2 DLL, session events will not be detected")
     log.error(" %s", e)
+    del e
     wtsapi32 = None
 NOTIFY_FOR_THIS_SESSION = 0
 
+KNOWN_EVENTS = {}
+POWER_EVENTS = {}
+for x in dir(win32con):
+    if x.endswith("_EVENT"):
+        v = getattr(win32con, x)
+        KNOWN_EVENTS[v] = x
+    if x.startswith("PBT_"):
+        v = getattr(win32con, x)
+        POWER_EVENTS[v] = x
+
 #no idea where we're supposed to get those from:
 WM_WTSSESSION_CHANGE        = 0x02b1
 WM_DWMNCRENDERINGCHANGED    = 0x031F
@@ -193,7 +204,7 @@ class Win32EventListener(object):
                     l = log.info
                 else:
                     ut = "/ unexpected"
-                l("unknown %s message: %s / %s / %s", ut, event_name, wParam, lParam)
+                l("unknown %s message: %s / %#x / %#x", ut, event_name, int(wParam), int(lParam))
         elif self.hwnd and hWnd!=None:
             log.warn("invalid hwnd: %s (expected %s)", hWnd, self.hwnd)
         if msg==win32con.WM_DESTROY:
