@@ -1,59 +1,20 @@
 # This file is part of Xpra.
-# Copyright (C) 2013-2017 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2013-2018 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 #authentication from a file containing a list of entries of the form:
 # username|password|uid|gid|displays|env_options|session_options
 
-import os
-import binascii
-import hmac
-
+from xpra.server.auth.sys_auth_base import parse_uid, parse_gid
 from xpra.server.auth.file_auth_base import log, FileAuthenticatorBase, init as file_init
-from xpra.os_util import strtobytes, POSIX
+from xpra.os_util import strtobytes, hexstr
 from xpra.util import parse_simple_dict
-from xpra.net.crypto import get_digest_module
+from xpra.net.crypto import verify_digest
 
 
-socket_dir = None
-socket_dirs = None
 def init(opts):
     file_init(opts)
-    global socket_dir, socket_dirs
-    socket_dir = opts.socket_dir
-    socket_dirs = opts.socket_dirs
-
-
-def getuid(v):
-    if v:
-        try:
-            return int(v)
-        except:
-            log("uid '%s' is not an int", v)
-    if POSIX:
-        try:
-            import pwd
-            return pwd.getpwnam(v or "nobody").pw_uid
-        except Exception as e:
-            log.error("Error: cannot find uid of '%s': %s", v, e)
-        return os.getuid()
-    return -1
-
-def getgid(v):
-    if v:
-        try:
-            return int(v)
-        except:
-            log("gid '%s' is not an int", v)
-    if POSIX:
-        try:
-            import grp          #@UnresolvedImport
-            return grp.getgrnam(v or "nobody").gr_gid
-        except Exception as e:
-            log.error("Error: cannot find gid of '%s': %s", v, e)
-        return os.getgid()
-    return -1
 
 
 def parse_auth_line(line):
@@ -64,11 +25,13 @@ def parse_auth_line(line):
     username = ldata[0]
     password = ldata[1]
     if len(ldata)>=5:
-        uid = getuid(ldata[2])
-        gid = getgid(ldata[3])
+        uid = parse_uid(ldata[2])
+        gid = parse_gid(ldata[3])
         displays = ldata[4].split(b",")
     else:
-        uid, gid = -1, -1
+        #this will use the default value, usually "nobody":
+        uid = parse_uid(None)
+        gid = parse_gid(None)
         displays = []
     env_options = {}
     session_options = {}
@@ -85,7 +48,6 @@ class Authenticator(FileAuthenticatorBase):
         self.sessions = None
 
     def parse_filedata(self, data):
-        global socket_dir, socket_dirs
         if not data:
             return {}
         auth_data = {}
@@ -126,8 +88,8 @@ class Authenticator(FileAuthenticatorBase):
             return None
         return entry[0]
 
-    def authenticate_hmac(self, challenge_response, client_salt):
-        log("authenticate_hmac(%s, %s)", challenge_response, client_salt)
+    def authenticate_hmac(self, challenge_response, client_salt=None):
+        log("authenticate_hmac(%r, %r)", challenge_response, client_salt)
         self.sessions = None
         if not self.salt:
             log.error("Error: illegal challenge response received - salt cleared or unset")
@@ -136,17 +98,14 @@ class Authenticator(FileAuthenticatorBase):
         salt = self.get_response_salt(client_salt)
         entry = self.get_auth_info()
         if entry is None:
-            log.error("Error: authentication failed")
-            log.error(" no password for '%s' in '%s'", self.username, self.password_filename)
+            log.warn("Warning: authentication failed")
+            log.warn(" no password for '%s' in '%s'", self.username, self.password_filename)
             return None
         log("authenticate: auth-info(%s)=%s", self.username, entry)
         fpassword, uid, gid, displays, env_options, session_options = entry
-        digestmod = get_digest_module(self.digest)
-        verify = hmac.HMAC(strtobytes(fpassword), strtobytes(salt), digestmod=digestmod).hexdigest()
-        log("multifile authenticate_hmac(%s) password='%s', hex(salt)=%s, hash=%s", challenge_response, fpassword, binascii.hexlify(strtobytes(salt)), verify)
-        if not hmac.compare_digest(verify, challenge_response):
-            log("expected '%s' but got '%s'", verify, challenge_response)
-            log.error("Error: hmac password challenge for '%s' does not match", self.username)
+        log("multifile authenticate_hmac password='%r', hex(salt)=%s", fpassword, hexstr(salt))
+        if not verify_digest(self.digest, fpassword, salt, challenge_response):
+            log.warn("Warning: %s challenge for '%s' does not match", self.digest, self.username)
             return False
         self.sessions = uid, gid, displays, env_options, session_options
         return True
