@@ -1,4 +1,4 @@
-# coding=utf8
+# -*- coding: utf-8 -*-
 # This file is part of Xpra.
 # Copyright (C) 2010-2016 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
@@ -14,7 +14,6 @@ class KeyboardConfigBase(object):
 
     def __init__(self):
         self.enabled = True
-        self.is_native_keymap = True
         self.owner = None
 
     def __repr__(self):
@@ -23,7 +22,6 @@ class KeyboardConfigBase(object):
     def get_info(self):
         return {
                 "enabled"   : self.enabled,
-                "native"    : self.is_native_keymap,
                 "owner"     : self.owner or "",
                 }
 
@@ -33,7 +31,7 @@ class KeyboardConfigBase(object):
     def get_hash(self):
         return ""
 
-    def set_layout(self, layout, variant):
+    def set_layout(self, layout, variant, options):
         pass
 
     def set_keymap(self, translate_only=False):
@@ -45,10 +43,10 @@ class KeyboardConfigBase(object):
     def make_keymask_match(self, modifier_list, ignored_modifier_keycode=None, ignored_modifier_keynames=None):
         pass
 
-    def get_keycode(self, client_keycode, keyname, modifiers):
+    def get_keycode(self, _client_keycode, _keyname, _modifiers):
         log("%s does not implement get_keycode!", type(self))
         return -1
 
-    def is_modifier(self, keycode):
+    def is_modifier(self, _keycode):
         #should be overriden in subclass
         return False
