@@ -43,7 +43,9 @@ try:
         from xpra.sound.pulseaudio import pulseaudio_pactl_util as _pulseaudio_util
 except ImportError as e:
     #fallback forks a process and parses the output:
+    log("cannot import default pulseaudio util: %s", e)
     log("using pulseaudio none fallback")
+    del e
     from xpra.sound.pulseaudio import pulseaudio_none_util as _pulseaudio_util
 
 get_info                = _pulseaudio_util.get_info
@@ -52,6 +54,7 @@ get_pa_device_options   = _pulseaudio_util.get_pa_device_options
 get_default_sink        = _pulseaudio_util.get_default_sink
 get_pulse_server        = _pulseaudio_util.get_pulse_server
 get_pulse_id            = _pulseaudio_util.get_pulse_id
+get_pulse_cookie_hash   = _pulseaudio_util.get_pulse_cookie_hash
 get_pactl_server        = _pulseaudio_util.get_pactl_server
 set_source_mute         = _pulseaudio_util.set_source_mute
 set_sink_mute           = _pulseaudio_util.set_sink_mute
