@@ -1,5 +1,5 @@
 # This file is part of Xpra.
-# Copyright (C) 2010-2016 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2010-2018 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -10,8 +10,6 @@ gstlog = Logger("gstreamer")
 #must be done before importing gobject!
 from xpra.sound.gstreamer_util import import_gst
 gst = import_gst()
-MESSAGE_ELEMENT = getattr(gst, "MESSAGE_ELEMENT", None)
-
 from xpra.util import envint
 from xpra.os_util import monotonic_time
 from xpra.gtk_common.gobject_compat import import_glib
@@ -55,7 +53,7 @@ class SoundPipeline(gobject.GObject):
         self.state = "stopped"
         self.buffer_count = 0
         self.byte_count = 0
-        self.emit_info_due = False
+        self.emit_info_timer = None
         self.info = {
                      "codec"        : self.codec,
                      "state"        : self.state,
@@ -69,17 +67,23 @@ class SoundPipeline(gobject.GObject):
         self.idle_add(self.emit, sig, *args)
 
     def emit_info(self):
-        if self.emit_info_due:
+        if self.emit_info_timer:
             return
-        self.emit_info_due = True
         def do_emit_info():
-            self.emit_info_due = False
+            self.emit_info_timer = None
             if self.pipeline:
                 info = self.get_info()
                 #reset info:
                 self.info = {}
                 self.emit("info", info)
-        self.timeout_add(200, do_emit_info)
+        self.emit_info_timer = self.timeout_add(200, do_emit_info)
+
+    def cancel_emit_info_timer(self):
+        eit = self.emit_info_timer
+        if eit:
+            self.emit_info_timer = None
+            self.source_remove(eit)
+
 
     def get_info(self):
         info = self.info.copy()
@@ -113,10 +117,10 @@ class SoundPipeline(gobject.GObject):
     def do_get_state(self, state):
         if not self.pipeline:
             return  "stopped"
-        return {gst.STATE_PLAYING   : "active",
-                gst.STATE_PAUSED    : "paused",
-                gst.STATE_NULL      : "stopped",
-                gst.STATE_READY     : "ready"}.get(state, "unknown")
+        return {gst.State.PLAYING   : "active",
+                gst.State.PAUSED    : "paused",
+                gst.State.NULL      : "stopped",
+                gst.State.READY     : "ready"}.get(state, "unknown")
 
     def get_state(self):
         return self.state
@@ -160,7 +164,7 @@ class SoundPipeline(gobject.GObject):
         log("SoundPipeline.start() codec=%s", self.codec)
         self.idle_emit("new-stream", self.codec)
         self.update_state("active")
-        self.pipeline.set_state(gst.STATE_PLAYING)
+        self.pipeline.set_state(gst.State.PLAYING)
         if self.stream_compressor:
             self.info["stream-compressor"] = self.stream_compressor
         self.emit_info()
@@ -193,11 +197,12 @@ class SoundPipeline(gobject.GObject):
         if self.state not in ("starting", "stopped", "ready", None):
             log.info("stopping")
         self.update_state("stopped")
-        p.set_state(gst.STATE_NULL)
+        p.set_state(gst.State.NULL)
         log("SoundPipeline.stop() done")
 
     def cleanup(self):
         log("SoundPipeline.cleanup()")
+        self.cancel_emit_info_timer()
         self.stop()
         b = self.bus
         self.bus = None
@@ -240,7 +245,7 @@ class SoundPipeline(gobject.GObject):
             return
         cdl = self.codec_description.lower()
         if not cdl or (cdl!=dl and dl.find(cdl)<0 and cdl.find(dl)<0):
-            self.gstloginfo("using audio codec %s", dl)
+            self.gstloginfo("using '%s' audio codec", dl)
         self.codec_description = dl
         self.info["codec_description"]  = dl
 
@@ -254,22 +259,22 @@ class SoundPipeline(gobject.GObject):
               "mpeg4"       : "iso fmp4",
               }.get(desc.lower(), desc.lower())
         if not cdl or (cdl!=dl and dl.find(cdl)<0 and cdl.find(dl)<0):
-            self.gstloginfo("using container format %s", dl)
+            self.gstloginfo("using '%s' container format", dl)
         self.container_description = dl
         self.info["container_description"]  = dl
 
 
-    def on_message(self, bus, message):
+    def on_message(self, _bus, message):
         #log("on_message(%s, %s)", bus, message)
         gstlog("on_message: %s", message)
         t = message.type
-        if t == gst.MESSAGE_EOS:
-            self.pipeline.set_state(gst.STATE_NULL)
+        if t == gst.MessageType.EOS:
+            self.pipeline.set_state(gst.State.NULL)
             self.gstloginfo("EOS")
             self.update_state("stopped")
             self.idle_emit("state-changed", self.state)
-        elif t == gst.MESSAGE_ERROR:
-            self.pipeline.set_state(gst.STATE_NULL)
+        elif t == gst.MessageType.ERROR:
+            self.pipeline.set_state(gst.State.NULL)
             err, details = message.parse_error()
             gstlog.error("pipeline error: %s", err)
             if self.pipeline_str:
@@ -289,40 +294,46 @@ class SoundPipeline(gobject.GObject):
             self.idle_emit("error", str(err))
             #exit
             self.cleanup()
-        elif t == gst.MESSAGE_TAG or t == MESSAGE_ELEMENT:
+        elif t == gst.MessageType.TAG:
             try:
                 self.parse_message(message)
             except Exception as e:
                 self.gstlogwarn("Warning: failed to parse gstreamer message:")
                 self.gstlogwarn(" %s: %s", type(e), e)
-        elif t == gst.MESSAGE_STREAM_STATUS:
+        elif t == gst.MessageType.ELEMENT:
+            try:
+                self.parse_element_message(message)
+            except Exception as e:
+                self.gstlogwarn("Warning: failed to parse gstreamer element message:")
+                self.gstlogwarn(" %s: %s", type(e), e)
+        elif t == gst.MessageType.STREAM_STATUS:
             gstlog("stream status: %s", message)
             try:
                 gstlog("stream status: %s", message.get_stream_status_object().get_state())
             except:
                 pass
-        elif t == gst.MESSAGE_STREAM_START:
+        elif t == gst.MessageType.STREAM_START:
             log("stream start: %s", message)
             #with gstreamer 1.x, we don't always get the "audio-codec" message..
             #so print the codec from here instead (and assume gstreamer is using what we told it to)
             #after a delay, just in case we do get the real "audio-codec" message!
             self.timeout_add(500, self.new_codec_description, self.codec.split("+")[0])
-        elif t in (gst.MESSAGE_ASYNC_DONE, gst.MESSAGE_NEW_CLOCK):
+        elif t in (gst.MessageType.ASYNC_DONE, gst.MessageType.NEW_CLOCK):
             gstlog("%s", message)
-        elif t == gst.MESSAGE_STATE_CHANGED:
+        elif t == gst.MessageType.STATE_CHANGED:
             _, new_state, _ = message.parse_state_changed()
-            gstlog("state-changed on %s: %s", message.src, gst.element_state_get_name(new_state))
+            gstlog("state-changed on %s: %s", message.src, gst.Element.state_get_name(new_state))
             state = self.do_get_state(new_state)
             if isinstance(message.src, gst.Pipeline):
                 self.update_state(state)
                 self.idle_emit("state-changed", state)
-        elif t == gst.MESSAGE_DURATION:
+        elif t == gst.MessageType.DURATION_CHANGED:
             gstlog("duration changed: %s", message)
-        elif t == gst.MESSAGE_LATENCY:
+        elif t == gst.MessageType.LATENCY:
             gstlog("latency message from %s: %s", message.src, message)
-        elif t == gst.MESSAGE_INFO:
+        elif t == gst.MessageType.INFO:
             self.gstloginfo("pipeline message: %s", message)
-        elif t == gst.MESSAGE_WARNING:
+        elif t == gst.MessageType.WARNING:
             w = message.parse_warning()
             self.gstlogwarn("pipeline warning: %s", w[0].message)
             for x in w[1:]:
@@ -331,7 +342,9 @@ class SoundPipeline(gobject.GObject):
                         if l.startswith("\n"):
                             l = l.strip("\n")+" "
                             for lp in l.split(". "):
-                                self.gstlogwarn(" %s", lp.strip())
+                                lp = lp.strip()
+                                if lp:
+                                    self.gstlogwarn(" %s", lp)
                         else:
                             self.gstlogwarn("                  %s", l.strip("\n\r"))
         else:
@@ -339,6 +352,19 @@ class SoundPipeline(gobject.GObject):
         self.emit_info()
         return 0
 
+    def parse_element_message(self, message):
+        structure = message.get_structure()
+        props = {
+            "seqnum"    : int(message.seqnum),
+            }
+        for i in range(structure.n_fields()):
+            name = structure.nth_field_name(i)
+            props[name] = structure.get_value(name)
+        self.do_parse_element_message(message, message.src.get_name(), props)
+
+    def do_parse_element_message(self, message, name, props={}):
+        gstlog("do_parse_element_message%s", (message, name, props))
+
     def parse_message(self, message):
         #message parsing code for GStreamer 1.x
         taglist = message.parse_tag()
@@ -380,4 +406,4 @@ class SoundPipeline(gobject.GObject):
             #no match yet
             if len([x for x in tags if x in ("minimum-bitrate", "maximum-bitrate", "channel-mode")])==0:
                 structure = message.get_structure()
-                self.gstloginfo("unknown sound pipeline tag message: %s", structure.to_string())
+                self.gstloginfo("unknown sound pipeline tag message: %s, tags=%s", structure.to_string(), tags)
