@@ -16,14 +16,16 @@ from xpra.x11.gtk_x11.send_wm import send_wm_take_focus
 from xpra.x11.gtk_x11.prop import prop_set, prop_get
 from xpra.x11.prop_conv import MotifWMHints
 from xpra.x11.bindings.window_bindings import X11WindowBindings #@UnresolvedImport
-from xpra.x11.gtk2 import Unmanageable
+from xpra.x11.common import Unmanageable
 from xpra.x11.gtk2.models.size_hints_util import sanitize_size_hints
-from xpra.x11.gtk2.models import MAX_WINDOW_SIZE
+from xpra.x11.common import MAX_WINDOW_SIZE
 from xpra.x11.gtk2.models.base import BaseWindowModel, constants
 from xpra.x11.gtk2.models.core import sanestr, gobject, xswallow, xsync
 from xpra.x11.gtk2.gdk_bindings import (
                 add_event_receiver,                         #@UnresolvedImport
                 remove_event_receiver,                      #@UnresolvedImport
+               )
+from xpra.gtk_common.gtk2.gdk_bindings import (
                 get_display_for,                            #@UnresolvedImport
                 calc_constrained_size,                      #@UnresolvedImport
                )
@@ -438,14 +440,14 @@ class WindowModel(BaseWindowModel):
             #try to honour initial size and position requests during setup:
             def window_size():
                 return self.get_property("requested-size")
-            def window_position(w=0, h=0):
+            def window_position(_w, _h):
                 return self.get_property("requested-position")
             geomlog("_update_client_geometry: using initial size=%s and position=%s", window_size(), window_position())
         else:
             geomlog("_update_client_geometry: ignored, owner=%s, setup_done=%s", owner, self._setup_done)
             def window_size():
                 return self.get_property("geometry")[2:4]
-            def window_position(w=0, h=0):
+            def window_position(_w, _h):
                 return self.get_property("geometry")[:2]
         self._do_update_client_geometry(window_size, window_position)
 
@@ -486,9 +488,11 @@ class WindowModel(BaseWindowModel):
             with xsync:
                 self.resize_corral_window(event.x, event.y, event.width, event.height, event.border_width)
         except XError as e:
-            geomlog.warn("failed to resize corral window: %s", e)
+            geomlog("do_xpra_configure_event(%s)", event, exc_info=True)
+            geomlog.warn("Warning: failed to resize corral window %#x", self.corral_window.xid)
+            geomlog.warn(" %s", e)
 
-    def resize_corral_window(self, x, y, w, h, border):
+    def resize_corral_window(self, x, y, w, h, _border):
         #the client window may have been resized or moved (generally programmatically)
         #so we may need to update the corral_window to match
         cox, coy, cow, coh = self.corral_window.get_geometry()[:4]
@@ -591,10 +595,10 @@ class WindowModel(BaseWindowModel):
 
     def _handle_icon_title_change(self):
         icon_name = self.prop_get("_NET_WM_ICON_NAME", "utf8", True)
-        metalog("_NET_WM_ICON_NAME=%s", icon_name)
+        iconlog("_NET_WM_ICON_NAME=%s", icon_name)
         if icon_name is None:
             icon_name = self.prop_get("WM_ICON_NAME", "latin1", True)
-            metalog("WM_ICON_NAME=%s", icon_name)
+            iconlog("WM_ICON_NAME=%s", icon_name)
         self._updateprop("icon-title", sanestr(icon_name))
 
     def _handle_motif_wm_hints_change(self):
@@ -663,7 +667,7 @@ class WindowModel(BaseWindowModel):
         #then get the icon pixels on demand and cache them..
         self._internal_set_property("icon", surf)
         self._internal_set_property("icon-pixmap", pixmap)
-        iconlog("icon is now %r", surf)
+        iconlog("icon is now %r, pixmap=%s", surf, pixmap)
 
     _x11_property_handlers = dict(BaseWindowModel._x11_property_handlers)
     _x11_property_handlers.update({
@@ -692,10 +696,13 @@ class WindowModel(BaseWindowModel):
             iconlog("%s.lookup_icon(%s)=%s", it, icon_name, i)
             if not i:
                 continue
-            p = i.load_icon()
-            iconlog("%s.load_icon()=%s", i, p)
-            if p:
-                break
+            try:
+                p = i.load_icon()
+                iconlog("%s.load_icon()=%s", i, p)
+                if p:
+                    break
+            except Exception:
+                iconlog("%s.load_icon()", i, exc_info=True)
         if p is None:
             return None
         #to make it consistent with the "icon" property,
